"""Sample of using the switch."""

from material_ui._component import Component, use_state
from material_ui.layout_basics import Stack
from material_ui.switch import Switch
from qtpy import QtWidgets
from qtpy.QtCore import Qt


class SampleSwitch(Component):
    """Window with two switches with linked state."""

    selected = use_state(False)

    def __init__(self) -> None:
        super().__init__()
        self.resize(300, 200)

        stack = Stack()
        stack.alignment = Qt.AlignmentFlag.AlignCenter
        stack.gap = 30
        stack.sx = {"background-color": "white"}
        self.overlay_widget(stack)

        switch = Switch()
        switch.selected = self.selected
        switch.on_change.connect(lambda value: setattr(self, "selected", value))
        stack.add_widget(switch)

        # This switch can't be toggled - value is controlled the other one.
        switch2 = Switch()
        switch2.selected = self.selected
        stack.add_widget(switch2)


def main() -> None:
    """Main function."""
    app = QtWidgets.QApplication()
    window = SampleSwitch()
    window.show()
    app.exec_()


if __name__ == "__main__":
    main()



"""Record gifs for docs.

Importing pyautogui locally as it breaks CI. Even though the record_gif
tests are skipped, this module has be imported to discover tests.
"""

import time
from collections.abc import Callable, Generator
from threading import Thread

import pytest
from pytestqt.qtbot import QtBot

from sample_buttons import SampleButtons
from sample_checkbox import SampleCheckbox
from sample_text_fields import SampleTextFields


class Controller(Thread):
    """Play back pyautogui actions in a separate thread."""

    def __init__(self) -> None:
        super().__init__()
        self.movements: list[Callable[[], None]] = []
        self.kill = False

    def run(self) -> None:
        """Execute movements."""
        while not self.kill:
            time.sleep(0.001)
            if self.movements:
                for movement in self.movements:
                    movement()
                return


@pytest.fixture
def controller() -> Generator[Controller, None, None]:
    thread = Controller()
    thread.start()
    yield thread
    thread.kill = True


def wait(duration_secs: float) -> Callable[[], None]:
    return lambda: time.sleep(duration_secs)


def click() -> None:
    import pyautogui

    pyautogui.mouseDown()
    time.sleep(0.2)
    pyautogui.mouseUp()


def move_to(x: int, y: int, *, instant: bool = False) -> Callable[[], None]:
    import pyautogui

    return lambda: pyautogui.moveTo(
        x,
        y,
        duration=0 if instant else 0.5,
        tween=pyautogui.easeInOutQuad,  # pyright: ignore[reportUnknownArgumentType]
    )


def typewrite(string: str) -> Callable[[], None]:
    import pyautogui

    return lambda: pyautogui.typewrite(string, interval=0.1)


def keypress(key: str) -> Callable[[], None]:
    import pyautogui

    return lambda: pyautogui.press(key)


@pytest.mark.record_gif
def test_sample_buttons_gif(qtbot: QtBot, controller: Controller) -> None:
    window = SampleButtons()
    qtbot.addWidget(window)
    window.show()
    with qtbot.wait_exposed(window):
        dpr = window.devicePixelRatioF()
        x = int(window.x() * dpr)
        y = int((window.y() + window.height() / 2 + 30) * dpr)
        controller.movements = [
            move_to(x - 50, y, instant=True),
            wait(1.5),
            move_to(x + (115 * dpr), y),
            click,
            move_to(x + (240 * dpr), y),
            click,
            move_to(x + (355 * dpr), y),
            click,
            move_to(x + (475 * dpr), y),
            click,
            move_to(x + (595 * dpr), y),
            click,
            move_to(x + (750 * dpr), y),
        ]
        qtbot.wait(12000)


@pytest.mark.record_gif
def test_sample_text_fields_gif(qtbot: QtBot, controller: Controller) -> None:
    window = SampleTextFields()
    qtbot.addWidget(window)
    window.show()
    with qtbot.wait_exposed(window):
        dpr = window.devicePixelRatioF()
        x = int(window.x() * dpr)
        y = int((window.y() + window.height() / 2 + 30) * dpr)
        controller.movements = [
            move_to(x + 200 * dpr, y + 100 * dpr, instant=True),
            wait(3),
            move_to(x + 200 * dpr, y),
            click,
            typewrite("Hello"),
            wait(0.75),
            move_to(x + 350 * dpr, y),
            click,
            wait(0.5),
            typewrite("world"),
            wait(1),
            move_to(x + 350 * dpr, y + 100 * dpr),
        ]
        qtbot.wait(14000)


@pytest.mark.record_gif
def test_sample_checkbox_gif(qtbot: QtBot, controller: Controller) -> None:
    window = SampleCheckbox()
    qtbot.addWidget(window)
    window.show()
    with qtbot.wait_exposed(window):
        dpr = window.devicePixelRatioF()
        x = int(window.x() * dpr)
        y = int((window.y() + window.height() / 2 + 30) * dpr)
        controller.movements = [
            move_to(x + 142 * dpr, y + 100 * dpr, instant=True),
            wait(2),
            move_to(x + 142 * dpr, y + 5),
            click,
            wait(1.5),
            click,
            wait(0.75),
            move_to(x + 142 * dpr, y + 100 * dpr),
        ]
        qtbot.wait(12000)



"""Sample of the dynamic color palette system."""

from dataclasses import dataclass, replace

from material_ui._component import Component, Signal, effect, use_state
from material_ui.layout_basics import Row, Stack
from material_ui.shape import Shape
from material_ui.switch import Switch
from material_ui.text_fields.filled_text_field import FilledTextField
from material_ui.theming.dynamic_color import apply_dynamic_color_scheme
from material_ui.tokens import md_sys_color
from material_ui.typography import Typography
from materialyoucolor.hct import Hct
from materialyoucolor.scheme.scheme_tonal_spot import SchemeTonalSpot
from qtpy.QtCore import QMargins, Qt
from qtpy.QtWidgets import QApplication, QGridLayout


class ColorGrid(Component):
    def __init__(self) -> None:
        super().__init__()

        self.sx = {"background-color": md_sys_color.background}

        self.setFixedWidth(1200)

        grid = QGridLayout()
        grid.setContentsMargins(QMargins(40, 40, 40, 40))
        grid.setAlignment(Qt.AlignmentFlag.AlignTop)
        grid.setSpacing(5)

        # TODO: cleanup repetitive quick code

        primary_cell = Shape()
        primary_cell.setFixedHeight(100)
        primary_cell.color = md_sys_color.primary
        primary_label = Typography()
        primary_label.variant = "label-large"
        primary_label.color = md_sys_color.on_primary
        primary_label.text = "Primary"
        primary_label.setParent(primary_cell)
        primary_label.move(10, 10)
        grid.addWidget(primary_cell, 0, 0, 1, 15)

        on_primary_cell = Shape()
        on_primary_cell.setFixedHeight(50)
        on_primary_cell.color = md_sys_color.on_primary
        on_primary_label = Typography()
        on_primary_label.variant = "label-large"
        on_primary_label.color = md_sys_color.primary
        on_primary_label.text = "On Primary"
        on_primary_label.setParent(on_primary_cell)
        on_primary_label.move(10, 10)
        grid.addWidget(on_primary_cell, 1, 0, 1, 15)

        secondary_cell = Shape()
        secondary_cell.setFixedHeight(100)
        secondary_cell.color = md_sys_color.secondary
        secondary_label = Typography()
        secondary_label.variant = "label-large"
        secondary_label.color = md_sys_color.on_secondary
        secondary_label.text = "Secondary"
        secondary_label.setParent(secondary_cell)
        secondary_label.move(10, 10)
        grid.addWidget(secondary_cell, 0, 15, 1, 15)

        on_secondary_cell = Shape()
        on_secondary_cell.setFixedHeight(50)
        on_secondary_cell.color = md_sys_color.on_secondary
        on_secondary_label = Typography()
        on_secondary_label.variant = "label-large"
        on_secondary_label.color = md_sys_color.secondary
        on_secondary_label.text = "On Secondary"
        on_secondary_label.setParent(on_secondary_cell)
        on_secondary_label.move(10, 10)
        grid.addWidget(on_secondary_cell, 1, 15, 1, 15)

        tertiary_cell = Shape()
        tertiary_cell.setFixedHeight(100)
        tertiary_cell.color = md_sys_color.tertiary
        tertiary_label = Typography()
        tertiary_label.variant = "label-large"
        tertiary_label.color = md_sys_color.on_tertiary
        tertiary_label.text = "Tertiary"
        tertiary_label.setParent(tertiary_cell)
        tertiary_label.move(10, 10)
        grid.addWidget(tertiary_cell, 0, 30, 1, 15)

        on_tertiary_cell = Shape()
        on_tertiary_cell.setFixedHeight(50)
        on_tertiary_cell.color = md_sys_color.on_tertiary
        on_tertiary_label = Typography()
        on_tertiary_label.variant = "label-large"
        on_tertiary_label.color = md_sys_color.tertiary
        on_tertiary_label.text = "On Tertiary"
        on_tertiary_label.setParent(on_tertiary_cell)
        on_tertiary_label.move(10, 10)
        grid.addWidget(on_tertiary_cell, 1, 30, 1, 15)

        error_cell = Shape()
        error_cell.setFixedHeight(100)
        error_cell.color = md_sys_color.error
        error_label = Typography()
        error_label.variant = "label-large"
        error_label.color = md_sys_color.on_error
        error_label.text = "Error"
        error_label.setParent(error_cell)
        error_label.move(10, 10)
        grid.addWidget(error_cell, 0, 45, 1, 15)

        on_error_cell = Shape()
        on_error_cell.setFixedHeight(50)
        on_error_cell.color = md_sys_color.on_error
        on_error_label = Typography()
        on_error_label.variant = "label-large"
        on_error_label.color = md_sys_color.error
        on_error_label.text = "On Error"
        on_error_label.setParent(on_error_cell)
        on_error_label.move(10, 10)
        grid.addWidget(on_error_cell, 1, 45, 1, 15)

        primary_container_cell = Shape()
        primary_container_cell.setFixedHeight(100)
        primary_container_cell.color = md_sys_color.primary_container
        primary_container_label = Typography()
        primary_container_label.variant = "label-large"
        primary_container_label.color = md_sys_color.on_primary_container
        primary_container_label.text = "Primary Container"
        primary_container_label.setParent(primary_container_cell)
        primary_container_label.move(10, 10)
        grid.addWidget(primary_container_cell, 2, 0, 1, 15)

        on_primary_container_cell = Shape()
        on_primary_container_cell.setFixedHeight(50)
        on_primary_container_cell.color = md_sys_color.on_primary_container
        on_primary_container_label = Typography()
        on_primary_container_label.variant = "label-large"
        on_primary_container_label.color = md_sys_color.primary_container
        on_primary_container_label.text = "On Primary Container"
        on_primary_container_label.setParent(on_primary_container_cell)
        on_primary_container_label.move(10, 10)
        grid.addWidget(on_primary_container_cell, 3, 0, 1, 15)

        secondary_container_cell = Shape()
        secondary_container_cell.setFixedHeight(100)
        secondary_container_cell.color = md_sys_color.secondary_container
        secondary_container_label = Typography()
        secondary_container_label.variant = "label-large"
        secondary_container_label.color = md_sys_color.on_secondary_container
        secondary_container_label.text = "Secondary Container"
        secondary_container_label.setParent(secondary_container_cell)
        secondary_container_label.move(10, 10)
        grid.addWidget(secondary_container_cell, 2, 15, 1, 15)

        on_secondary_container_cell = Shape()
        on_secondary_container_cell.setFixedHeight(50)
        on_secondary_container_cell.color = md_sys_color.on_secondary_container
        on_secondary_container_label = Typography()
        on_secondary_container_label.variant = "label-large"
        on_secondary_container_label.color = md_sys_color.secondary_container
        on_secondary_container_label.text = "On Secondary Container"
        on_secondary_container_label.setParent(on_secondary_container_cell)
        on_secondary_container_label.move(10, 10)
        grid.addWidget(on_secondary_container_cell, 3, 15, 1, 15)

        tertiary_container_cell = Shape()
        tertiary_container_cell.setFixedHeight(100)
        tertiary_container_cell.color = md_sys_color.tertiary_container
        tertiary_container_label = Typography()
        tertiary_container_label.variant = "label-large"
        tertiary_container_label.color = md_sys_color.on_tertiary_container
        tertiary_container_label.text = "Tertiary Container"
        tertiary_container_label.setParent(tertiary_container_cell)
        tertiary_container_label.move(10, 10)
        grid.addWidget(tertiary_container_cell, 2, 30, 1, 15)

        on_tertiary_container_cell = Shape()
        on_tertiary_container_cell.setFixedHeight(50)
        on_tertiary_container_cell.color = md_sys_color.on_tertiary_container
        on_tertiary_container_label = Typography()
        on_tertiary_container_label.variant = "label-large"
        on_tertiary_container_label.color = md_sys_color.tertiary_container
        on_tertiary_container_label.text = "On Tertiary Container"
        on_tertiary_container_label.setParent(on_tertiary_container_cell)
        on_tertiary_container_label.move(10, 10)
        grid.addWidget(on_tertiary_container_cell, 3, 30, 1, 15)

        error_container_cell = Shape()
        error_container_cell.setFixedHeight(100)
        error_container_cell.color = md_sys_color.error_container
        error_container_label = Typography()
        error_container_label.variant = "label-large"
        error_container_label.color = md_sys_color.on_error_container
        error_container_label.text = "Error Container"
        error_container_label.setParent(error_container_cell)
        error_container_label.move(10, 10)
        grid.addWidget(error_container_cell, 2, 45, 1, 15)

        on_error_container_cell = Shape()
        on_error_container_cell.setFixedHeight(50)
        on_error_container_cell.color = md_sys_color.on_error_container
        on_error_container_label = Typography()
        on_error_container_label.variant = "label-large"
        on_error_container_label.color = md_sys_color.error_container
        on_error_container_label.text = "On Error Container"
        on_error_container_label.setParent(on_error_container_cell)
        on_error_container_label.move(10, 10)
        grid.addWidget(on_error_container_cell, 3, 45, 1, 15)

        surface_dim_cell = Shape()
        surface_dim_cell.setFixedHeight(100)
        surface_dim_cell.color = md_sys_color.surface_dim
        surface_dim_label = Typography()
        surface_dim_label.variant = "label-large"
        surface_dim_label.color = md_sys_color.on_surface
        surface_dim_label.text = "Surface Dim"
        surface_dim_label.setParent(surface_dim_cell)
        surface_dim_label.move(10, 10)
        grid.addWidget(surface_dim_cell, 4, 0, 1, 20)

        surface_cell = Shape()
        surface_cell.setFixedHeight(100)
        surface_cell.color = md_sys_color.surface
        surface_label = Typography()
        surface_label.variant = "label-large"
        surface_label.color = md_sys_color.on_surface
        surface_label.text = "Surface"
        surface_label.setParent(surface_cell)
        surface_label.move(10, 10)
        grid.addWidget(surface_cell, 4, 20, 1, 20)

        surface_bright_cell = Shape()
        surface_bright_cell.setFixedHeight(100)
        surface_bright_cell.color = md_sys_color.surface_bright
        surface_bright_label = Typography()
        surface_bright_label.variant = "label-large"
        surface_bright_label.color = md_sys_color.on_surface
        surface_bright_label.text = "Surface Bright"
        surface_bright_label.setParent(surface_bright_cell)
        surface_bright_label.move(10, 10)
        grid.addWidget(surface_bright_cell, 4, 40, 1, 20)

        surface_container_lowest_cell = Shape()
        surface_container_lowest_cell.setFixedHeight(100)
        surface_container_lowest_cell.color = md_sys_color.surface_container_lowest
        surface_container_lowest_label = Typography()
        surface_container_lowest_label.variant = "label-large"
        surface_container_lowest_label.color = md_sys_color.on_surface
        surface_container_lowest_label.text = "Surface Container Lowest"
        surface_container_lowest_label.setParent(surface_container_lowest_cell)
        surface_container_lowest_label.move(10, 10)
        grid.addWidget(surface_container_lowest_cell, 5, 0, 1, 12)

        surface_container_low_cell = Shape()
        surface_container_low_cell.setFixedHeight(100)
        surface_container_low_cell.color = md_sys_color.surface_container_low
        surface_container_low_label = Typography()
        surface_container_low_label.variant = "label-large"
        surface_container_low_label.color = md_sys_color.on_surface
        surface_container_low_label.text = "Surface Container Low"
        surface_container_low_label.setParent(surface_container_low_cell)
        surface_container_low_label.move(10, 10)
        grid.addWidget(surface_container_low_cell, 5, 12, 1, 12)

        surface_container_cell = Shape()
        surface_container_cell.setFixedHeight(100)
        surface_container_cell.color = md_sys_color.surface_container
        surface_container_label = Typography()
        surface_container_label.variant = "label-large"
        surface_container_label.color = md_sys_color.on_surface
        surface_container_label.text = "Surface Container"
        surface_container_label.setParent(surface_container_cell)
        surface_container_label.move(10, 10)
        grid.addWidget(surface_container_cell, 5, 24, 1, 12)

        surface_container_high_cell = Shape()
        surface_container_high_cell.setFixedHeight(100)
        surface_container_high_cell.color = md_sys_color.surface_container_high
        surface_container_high_label = Typography()
        surface_container_high_label.variant = "label-large"
        surface_container_high_label.color = md_sys_color.on_surface
        surface_container_high_label.text = "Surface Container High"
        surface_container_high_label.setParent(surface_container_high_cell)
        surface_container_high_label.move(10, 10)
        grid.addWidget(surface_container_high_cell, 5, 36, 1, 12)

        surface_container_highest_cell = Shape()
        surface_container_highest_cell.setFixedHeight(100)
        surface_container_highest_cell.color = md_sys_color.surface_container_highest
        surface_container_highest_label = Typography()
        surface_container_highest_label.variant = "label-large"
        surface_container_highest_label.color = md_sys_color.on_surface
        surface_container_highest_label.text = "Surface Container Highest"
        surface_container_highest_label.setParent(surface_container_highest_cell)
        surface_container_highest_label.move(10, 10)
        grid.addWidget(surface_container_highest_cell, 5, 48, 1, 12)

        on_surface_cell = Shape()
        on_surface_cell.setFixedHeight(60)
        on_surface_cell.color = md_sys_color.on_surface
        on_surface_label = Typography()
        on_surface_label.variant = "label-large"
        on_surface_label.color = md_sys_color.surface
        on_surface_label.text = "On Surface"
        on_surface_label.setParent(on_surface_cell)
        on_surface_label.move(10, 10)
        grid.addWidget(on_surface_cell, 6, 0, 1, 15)

        on_surface_variant_cell = Shape()
        on_surface_variant_cell.setFixedHeight(60)
        on_surface_variant_cell.color = md_sys_color.on_surface_variant
        on_surface_variant_label = Typography()
        on_surface_variant_label.variant = "label-large"
        on_surface_variant_label.color = md_sys_color.surface_variant
        on_surface_variant_label.text = "On Surface Variant"
        on_surface_variant_label.setParent(on_surface_variant_cell)
        on_surface_variant_label.move(10, 10)
        grid.addWidget(on_surface_variant_cell, 6, 15, 1, 15)

        outline_cell = Shape()
        outline_cell.setFixedHeight(60)
        outline_cell.color = md_sys_color.outline
        outline_label = Typography()
        outline_label.variant = "label-large"
        outline_label.color = md_sys_color.surface_variant
        outline_label.text = "Outline"
        outline_label.setParent(outline_cell)
        outline_label.move(10, 10)
        grid.addWidget(outline_cell, 6, 30, 1, 15)

        outline_variant_cell = Shape()
        outline_variant_cell.setFixedHeight(60)
        outline_variant_cell.color = md_sys_color.outline_variant
        outline_variant_label = Typography()
        outline_variant_label.variant = "label-large"
        outline_variant_label.color = md_sys_color.on_surface
        outline_variant_label.text = "Outline Variant"
        outline_variant_label.setParent(outline_variant_cell)
        outline_variant_label.move(10, 10)
        grid.addWidget(outline_variant_cell, 6, 45, 1, 15)

        self.setLayout(grid)


@dataclass
class Settings:
    color_hex: str = "#4181EE"
    is_dark: bool = False


class SettingsSideBar(Component):
    settings = use_state(Settings())
    on_change_settings: Signal[Settings]

    def __init__(self) -> None:
        super().__init__()

        self.sx = {"background-color": md_sys_color.surface}

        stack = Stack()
        stack.alignment = Qt.AlignmentFlag.AlignTop
        stack.gap = 15
        stack.margins = QMargins(20, 20, 20, 20)

        title = Typography()
        title.variant = "headline-medium"
        title.text = "Color Palette"
        title.color = md_sys_color.on_surface
        stack.add_widget(title)

        self._color_hex_textfield = FilledTextField()
        self._color_hex_textfield.label = "Color (Hex)"
        self._color_hex_textfield.on_change.connect(self._on_change_color_hex)
        stack.add_widget(self._color_hex_textfield)

        dark_mode_row = Row()
        dark_mode_row.gap = 5
        dark_mode_label = Typography()
        dark_mode_label.variant = "body-large"
        dark_mode_label.text = "Dark Mode"
        dark_mode_label.color = md_sys_color.on_surface
        dark_mode_label.alignment = Qt.AlignmentFlag.AlignVCenter
        dark_mode_row.add_widget(dark_mode_label)
        self._dark_mode_switch = Switch()
        self._dark_mode_switch.on_change.connect(self._on_change_dark_mode)
        dark_mode_row.add_widget(self._dark_mode_switch)
        stack.add_widget(dark_mode_row)

        self.overlay_widget(stack)

    @effect(settings)
    def _apply_state(self) -> None:
        self._dark_mode_switch.selected = self.settings.is_dark
        self._color_hex_textfield.value = self.settings.color_hex

    def _on_change_dark_mode(self, selected: bool) -> None:  # noqa: FBT001
        new_state = replace(self.settings, is_dark=selected)
        self.on_change_settings.emit(new_state)

    def _on_change_color_hex(self, value: str) -> None:
        new_state = replace(self.settings, color_hex=value)
        self.on_change_settings.emit(new_state)


class DemoColorPalette(Component):
    settings = use_state(Settings())

    def __init__(self) -> None:
        super().__init__()

        # Clear the focus when clicking outside any input widget.
        self.clicked.connect(lambda: self.setFocus())

        row = Row()

        color_grid = ColorGrid()
        row.add_widget(color_grid)

        side_bar = SettingsSideBar()
        side_bar.settings = self.settings
        side_bar.on_change_settings.connect(self.set_state("settings"))
        row.add_widget(side_bar)

        self.overlay_widget(row)

    @effect(settings)
    def _apply_dynamic_color_scheme(self) -> None:
        color_hex = self.settings.color_hex
        is_dark = self.settings.is_dark
        scheme = SchemeTonalSpot(
            Hct.from_int(int(color_hex.replace("#", "0xFF"), 16)),
            is_dark=is_dark,
            contrast_level=0.0,
        )
        apply_dynamic_color_scheme(scheme)


def main() -> None:
    app = QApplication()
    window = DemoColorPalette()
    window.show()
    app.exec_()


if __name__ == "__main__":
    main()



from pytestqt.qtbot import QtBot

from material_ui.text_fields import FilledTextField


def test_FilledTextField_basic_api(qtbot: QtBot):
    text_field = FilledTextField()
    text_field.label = "Label"
    text_field.value = "Value"
    qtbot.addWidget(text_field)



"""Sample usage of Text Field components."""

from material_ui import Component
from material_ui.layout_basics import Row
from material_ui.text_fields import FilledTextField, OutlinedTextField
from material_ui.tokens import md_sys_color
from qtpy.QtCore import QMargins
from qtpy.QtWidgets import QApplication


class SampleTextFields(Component):
    """Sample usage of Text Field components."""

    def __init__(self) -> None:
        super().__init__()

        self.sx = {"background-color": md_sys_color.background}

        row = Row()
        row.gap = 30
        row.margins = QMargins(40, 30, 40, 30)
        self.overlay_widget(row)

        filled = FilledTextField()
        filled.label = "Filled"
        filled.value = ""
        row.add_widget(filled)

        outlined = OutlinedTextField()
        outlined.label = "Outlined"
        outlined.value = ""
        row.add_widget(outlined)

        # Take the initial focus away from the first text field.
        self.setFocus()
        # Steal focus when the empty area is clicked.
        self.clicked.connect(lambda: self.setFocus())


def main() -> None:
    app = QApplication()
    window = SampleTextFields()
    window.show()
    app.exec_()


if __name__ == "__main__":
    main()



import pytest
from materialyoucolor.hct import Hct
from materialyoucolor.scheme.scheme_tonal_spot import SchemeTonalSpot
from qtpy.QtGui import QColor

from material_ui.theming.dynamic_color import apply_dynamic_color_scheme
from material_ui.tokens import md_sys_color
from material_ui.tokens._utils import resolve_token


# Mark it last as it modifies global state. May clean this up later.
@pytest.mark.order("last")
def test_apply_dynamic_color_scheme_basic():
    light_blue_scheme = SchemeTonalSpot(
        Hct.from_int(0xFF4181EE),
        is_dark=False,
        contrast_level=0.0,
    )
    apply_dynamic_color_scheme(light_blue_scheme)
    assert resolve_token(md_sys_color.primary) == QColor("#445e91")
    assert resolve_token(md_sys_color.surface) == QColor("#f9f9ff")

    dark_pink_scheme = SchemeTonalSpot(
        Hct.from_int(0xFFD224B5),
        is_dark=True,
        contrast_level=0.0,
    )
    apply_dynamic_color_scheme(dark_pink_scheme)
    assert resolve_token(md_sys_color.primary) == QColor("#f6b2df")
    assert resolve_token(md_sys_color.surface) == QColor("#181215")



"""Linear progress indicator."""

from functools import partial
from typing import cast

from qtpy.QtCore import QEasingCurve, QPointF, QRect, Qt, QVariantAnimation
from qtpy.QtGui import QColor, QPainter, QPaintEvent
from qtpy.QtWidgets import QSizePolicy

from material_ui._component import effect, use_state
from material_ui.progress_indicators._base_progress import BaseProgress
from material_ui.tokens import md_comp_linear_progress_indicator as tokens
from material_ui.tokens._utils import resolve_token

_INDETERMINATE_ANIM_DURATION = 2000
_INDETERMINATE_ANIM_EASING = QEasingCurve(QEasingCurve.Type.BezierSpline)
_INDETERMINATE_ANIM_EASING.addCubicBezierSegment(
    QPointF(0.2, 0.3),
    QPointF(0.9, 1.0),
    QPointF(1.0, 1.0),
)


class LinearProgress(BaseProgress):
    """Linear progress indicator component."""

    _bar1_rect = use_state(QRect())
    _bar2_rect = use_state(QRect())

    _bar1_translate = use_state(0.0)
    _bar1_scale = use_state(0.0)
    _bar2_translate = use_state(0.0)
    _bar2_scale = use_state(0.0)

    def __init__(self) -> None:
        super().__init__()
        self.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Fixed)
        self.setFixedHeight(cast("int", resolve_token(tokens.track_height)))

    @effect(BaseProgress.value, BaseProgress.indeterminate)
    def _apply_bar_geometry(self) -> None:
        if not self.indeterminate:
            self._bar1_rect = QRect(0, 0, int(self.value * self.width()), self.height())
            self._bar2_rect = QRect()
        else:
            # Magic numbers from material-web implementation:
            # https://github.com/material-components/material-web/blob/main/progress/internal/_linear-progress.scss#L141
            anim = QVariantAnimation()
            anim.setParent(self)
            anim.setStartValue(0.0 - 1.45167)
            anim.setKeyValueAt(0.4, 0.0 - 1.45167)
            anim.setKeyValueAt(0.5515, 0.836714 - 1.45167)
            anim.setEndValue(2.00611 - 1.45167)
            anim.setDuration(_INDETERMINATE_ANIM_DURATION)
            anim.setEasingCurve(_INDETERMINATE_ANIM_EASING)
            anim.setLoopCount(-1)
            anim.valueChanged.connect(partial(setattr, self, "_bar1_translate"))
            anim.start()

            anim = QVariantAnimation()
            anim.setParent(self)
            anim.setStartValue(0.08)
            anim.setKeyValueAt(0.3665, 0.08)
            anim.setKeyValueAt(0.6915, 0.561479)
            anim.setKeyValueAt(0.93, 0.08)
            anim.setEndValue(0.08)
            anim.setDuration(_INDETERMINATE_ANIM_DURATION)
            anim.setEasingCurve(_INDETERMINATE_ANIM_EASING)
            anim.setLoopCount(-1)
            anim.valueChanged.connect(partial(setattr, self, "_bar1_scale"))
            anim.start()

            anim = QVariantAnimation()
            anim.setParent(self)
            anim.setStartValue(0.0 - 0.548889)
            anim.setKeyValueAt(0.25, 0.376519 - 0.548889)
            anim.setKeyValueAt(0.4835, 0.843862 - 0.548889)
            anim.setEndValue(1.60278 - 0.548889)
            anim.setDuration(_INDETERMINATE_ANIM_DURATION)
            anim.setEasingCurve(_INDETERMINATE_ANIM_EASING)
            anim.setLoopCount(-1)
            anim.valueChanged.connect(partial(setattr, self, "_bar2_translate"))
            anim.start()

            anim = QVariantAnimation()
            anim.setParent(self)
            anim.setStartValue(0.08)
            anim.setKeyValueAt(0.1915, 0.457104)
            anim.setKeyValueAt(0.4415, 0.72796)
            anim.setEndValue(0.08)
            anim.setDuration(_INDETERMINATE_ANIM_DURATION)
            anim.setEasingCurve(_INDETERMINATE_ANIM_EASING)
            anim.setLoopCount(-1)
            anim.valueChanged.connect(partial(setattr, self, "_bar2_scale"))
            anim.start()

    @effect(_bar1_translate, _bar1_scale, _bar2_translate, _bar2_scale)
    def _apply_animated_bar_geometry(self) -> None:
        w = self.width()
        h = self.height()

        # Condense 2 nested rects into 1 by offsetting the position so
        # that the inner rect would appear centered.
        self._bar1_rect = QRect(
            int((self._bar1_translate + (1 - self._bar1_scale) / 2) * w),
            0,
            int(self._bar1_scale * w),
            h,
        )

        self._bar2_rect = QRect(
            int((self._bar2_translate + (1 - self._bar2_scale) / 2) * w),
            0,
            int(self._bar2_scale * w),
            h,
        )

    @effect(_bar1_rect, _bar2_rect)
    def _update_on_rect_change(self) -> None:
        self.update()

    def paintEvent(self, event: QPaintEvent) -> None:  # noqa: N802
        """Overridden QWidget.paintEvent."""
        super().paintEvent(event)

        painter = QPainter(self)
        painter.setPen(Qt.PenStyle.NoPen)

        painter.setBrush(cast("QColor", resolve_token(tokens.track_color)))
        painter.drawRect(self.rect())

        painter.setBrush(cast("QColor", resolve_token(tokens.active_indicator_color)))
        painter.drawRect(self._bar1_rect)
        painter.drawRect(self._bar2_rect)

        painter.end()



"""Design tokens for md.comp.filled-text-field."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

active_indicator_color = _define_token("md.sys.color.on-surface-variant")
active_indicator_height = _define_token(1)
caret_color = _define_token("md.sys.color.primary")
container_color = _define_token("md.sys.color.surface-container-highest")
container_height = _define_token(56)
container_shape = _define_token("md.sys.shape.corner.extra-small.top")
disabled_active_indicator_color = _define_token("md.sys.color.on-surface")
disabled_active_indicator_height = _define_token(1)
disabled_active_indicator_opacity = _define_token(0.38)
disabled_container_color = _define_token("md.sys.color.on-surface")
disabled_container_opacity = _define_token(0.04)
disabled_input_text_color = _define_token("md.sys.color.on-surface")
disabled_input_text_opacity = _define_token(0.38)
disabled_label_text_color = _define_token("md.sys.color.on-surface")
disabled_label_text_opacity = _define_token(0.38)
disabled_leading_icon_color = _define_token("md.sys.color.on-surface")
disabled_leading_icon_opacity = _define_token(0.38)
disabled_supporting_text_color = _define_token("md.sys.color.on-surface")
disabled_supporting_text_opacity = _define_token(0.38)
disabled_trailing_icon_color = _define_token("md.sys.color.on-surface")
disabled_trailing_icon_opacity = _define_token(0.38)
error_active_indicator_color = _define_token("md.sys.color.error")
error_focus_active_indicator_color = _define_token("md.sys.color.error")
error_focus_caret_color = _define_token("md.sys.color.error")
error_focus_input_text_color = _define_token("md.sys.color.on-surface")
error_focus_label_text_color = _define_token("md.sys.color.error")
error_focus_leading_icon_color = _define_token("md.sys.color.on-surface-variant")
error_focus_supporting_text_color = _define_token("md.sys.color.error")
error_focus_trailing_icon_color = _define_token("md.sys.color.error")
error_hover_active_indicator_color = _define_token("md.sys.color.on-error-container")
error_hover_input_text_color = _define_token("md.sys.color.on-surface")
error_hover_label_text_color = _define_token("md.sys.color.on-error-container")
error_hover_leading_icon_color = _define_token("md.sys.color.on-surface-variant")
error_hover_state_layer_color = _define_token("md.sys.color.on-surface")
error_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
error_hover_supporting_text_color = _define_token("md.sys.color.error")
error_hover_trailing_icon_color = _define_token("md.sys.color.on-error-container")
error_input_text_color = _define_token("md.sys.color.on-surface")
error_label_text_color = _define_token("md.sys.color.error")
error_leading_icon_color = _define_token("md.sys.color.on-surface-variant")
error_supporting_text_color = _define_token("md.sys.color.error")
error_trailing_icon_color = _define_token("md.sys.color.error")
focus_active_indicator_color = _define_token("md.sys.color.primary")
focus_active_indicator_height = _define_token(2)
focus_active_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
focus_input_text_color = _define_token("md.sys.color.on-surface")
focus_label_text_color = _define_token("md.sys.color.primary")
focus_leading_icon_color = _define_token("md.sys.color.on-surface-variant")
focus_supporting_text_color = _define_token("md.sys.color.on-surface-variant")
focus_trailing_icon_color = _define_token("md.sys.color.on-surface-variant")
hover_active_indicator_color = _define_token("md.sys.color.on-surface")
hover_active_indicator_height = _define_token(1)
hover_input_text_color = _define_token("md.sys.color.on-surface")
hover_label_text_color = _define_token("md.sys.color.on-surface-variant")
hover_leading_icon_color = _define_token("md.sys.color.on-surface-variant")
hover_state_layer_color = _define_token("md.sys.color.on-surface")
hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
hover_supporting_text_color = _define_token("md.sys.color.on-surface-variant")
hover_trailing_icon_color = _define_token("md.sys.color.on-surface-variant")
input_text_color = _define_token("md.sys.color.on-surface")
input_text_font = _define_token("md.sys.typescale.body-large.font")
input_text_line_height = _define_token("md.sys.typescale.body-large.line-height")
input_text_placeholder_color = _define_token("md.sys.color.on-surface-variant")
input_text_prefix_color = _define_token("md.sys.color.on-surface-variant")
input_text_size = _define_token("md.sys.typescale.body-large.size")
input_text_suffix_color = _define_token("md.sys.color.on-surface-variant")
input_text_tracking = _define_token("md.sys.typescale.body-large.tracking")
input_text_weight = _define_token("md.sys.typescale.body-large.weight")
label_text_color = _define_token("md.sys.color.on-surface-variant")
label_text_font = _define_token("md.sys.typescale.body-large.font")
label_text_line_height = _define_token("md.sys.typescale.body-large.line-height")
label_text_populated_line_height = _define_token("md.sys.typescale.body-small.line-height")
label_text_populated_size = _define_token("md.sys.typescale.body-small.size")
label_text_size = _define_token("md.sys.typescale.body-large.size")
label_text_tracking = _define_token("md.sys.typescale.body-large.tracking")
label_text_weight = _define_token("md.sys.typescale.body-large.weight")
leading_icon_color = _define_token("md.sys.color.on-surface-variant")
leading_icon_size = _define_token(24)
supporting_text_color = _define_token("md.sys.color.on-surface-variant")
supporting_text_font = _define_token("md.sys.typescale.body-small.font")
supporting_text_line_height = _define_token("md.sys.typescale.body-small.line-height")
supporting_text_size = _define_token("md.sys.typescale.body-small.size")
supporting_text_tracking = _define_token("md.sys.typescale.body-small.tracking")
supporting_text_weight = _define_token("md.sys.typescale.body-small.weight")
trailing_icon_color = _define_token("md.sys.color.on-surface-variant")
trailing_icon_size = _define_token(24)



"""Base class for Text Field components."""

from typing import Literal, cast

from qtpy.QtCore import QPoint, QSize, Qt
from qtpy.QtWidgets import QLineEdit, QSizePolicy

from material_ui._component import Component, Signal, effect, use_state
from material_ui.tokens import md_comp_filled_text_field as tokens
from material_ui.tokens._utils import resolve_token
from material_ui.typography import Typography

LabelState = Literal["resting", "floating"]


class BaseTextField(Component):
    """Base class for Text Field components."""

    label = use_state("")
    """Floating label text."""

    value = use_state("")
    """Current value of the text field."""

    on_change: Signal[str]
    """Emitted when the value changed."""

    def __init__(self) -> None:
        super().__init__()

        self.setCursor(Qt.CursorShape.IBeamCursor)
        self.setSizePolicy(
            QSizePolicy.Policy.Expanding,
            QSizePolicy.Policy.Fixed,
        )

        self._resting_label = Typography()
        self._resting_label.text = self.label
        self._resting_label.setAttribute(
            Qt.WidgetAttribute.WA_TransparentForMouseEvents,
        )

        self._floating_label = Typography()
        self._floating_label.text = self.label
        self._floating_label.setAttribute(
            Qt.WidgetAttribute.WA_TransparentForMouseEvents,
        )

        self._line_edit = QLineEdit()
        # Disable Qt's default context menu.
        self._line_edit.setContextMenuPolicy(Qt.ContextMenuPolicy.NoContextMenu)
        self._line_edit.textEdited.connect(self._on_line_edit_text_edited)
        # Focus pass through to the line edit.
        self.setFocusProxy(self._line_edit)

        self.clicked.connect(self._on_clicked)
        self.should_propagate_click = False

    _TOP_SPACE = 6

    def sizeHint(self) -> QSize:  # noqa: N802
        height = cast("int", resolve_token(tokens.container_height))
        return QSize(200, height + self._TOP_SPACE)

    def _on_clicked(self) -> None:
        # Focus the itself when clicked.
        self.setFocus(Qt.FocusReason.MouseFocusReason)
        # Select all text in the line edit.
        self._line_edit.setSelection(0, len(self.value))

    def _on_line_edit_text_edited(self, text: str) -> None:
        # Set the internal value already for non-controlled text fields.
        # However the controlled case would also set it again.
        self.value = text
        self.on_change.emit(text)

    @effect(value)
    def _apply_value(self) -> None:
        cursor_pos = self._line_edit.cursorPosition()
        self._line_edit.setText(self.value)
        # Ensure cursor position is maintained.
        self._line_edit.setCursorPosition(cursor_pos)

    _label_state = use_state(cast("LabelState", "resting"))

    @effect(value, Component.focused)
    def _update_label_state(self) -> None:
        self._label_state = "floating" if self.value or self.focused else "resting"

    # Configured in derived classes.
    _RESTING_LABEL_POS = QPoint()
    _FLOATING_LABEL_POS = QPoint()

    _resting_label_opacity = use_state(0.0, transition=80)
    _floating_label_opacity = use_state(0.0, transition=80)
    _resting_label_pos = use_state(QPoint(), transition=80)
    _floating_label_pos = use_state(QPoint(), transition=80)

    @effect(_label_state)
    def _animate_labels(self) -> None:
        # Animate the positions and opacities. Instead of animating font
        # size, animate the opacity and positions to make two labels
        # look like one.
        match self._label_state:
            case "resting":
                self._resting_label_opacity = 1.0
                self._floating_label_opacity = 0.0
                self._resting_label_pos = self._RESTING_LABEL_POS
                self._floating_label_pos = self._RESTING_LABEL_POS
            case "floating":
                self._resting_label_opacity = 0.0
                self._floating_label_opacity = 1.0
                self._resting_label_pos = self._FLOATING_LABEL_POS
                self._floating_label_pos = self._FLOATING_LABEL_POS

    @effect(_resting_label_opacity)
    def _apply_resting_label_opacity(self) -> None:
        self._resting_label.sx = {
            **self._resting_label.sx,
            "opacity": self._resting_label_opacity,
        }

    @effect(_floating_label_opacity)
    def _apply_floating_label_opacity(self) -> None:
        self._floating_label.sx = {
            **self._floating_label.sx,
            "opacity": self._floating_label_opacity,
        }

    @effect(_resting_label_pos)
    def _apply_resting_label_pos(self) -> None:
        self._resting_label.move(self._resting_label_pos)

    @effect(_floating_label_pos)
    def _apply_floating_label_pos(self) -> None:
        self._floating_label.move(self._floating_label_pos)



"""Design tokens for md.comp.primary-navigation-tab."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

active_indicator_color = _define_token("md.sys.color.primary")
active_indicator_height = _define_token(3)
active_indicator_shape = _define_token("SHAPE_FAMILY_ROUNDED_CORNERS")
active_focus_state_layer_color = _define_token("md.sys.color.primary")
active_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
active_hover_state_layer_color = _define_token("md.sys.color.primary")
active_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
active_pressed_state_layer_color = _define_token("md.sys.color.primary")
active_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
container_color = _define_token("md.sys.color.surface")
container_elevation = _define_token("md.sys.elevation.level0")
container_height = _define_token(48)
container_shape = _define_token("md.sys.shape.corner.none")
divider_color = _define_token("md.sys.color.surface-variant")
divider_height = _define_token(1)
focus_indicator_color = _define_token("md.sys.color.secondary")
focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.inner-offset")
focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
inactive_focus_state_layer_color = _define_token("md.sys.color.on-surface")
inactive_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
inactive_hover_state_layer_color = _define_token("md.sys.color.on-surface")
inactive_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
inactive_pressed_state_layer_color = _define_token("md.sys.color.primary")
inactive_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
with_icon_and_label_text_container_height = _define_token(64)
with_icon_active_focus_icon_color = _define_token("md.sys.color.primary")
with_icon_active_hover_icon_color = _define_token("md.sys.color.primary")
with_icon_active_icon_color = _define_token("md.sys.color.primary")
with_icon_active_pressed_icon_color = _define_token("md.sys.color.primary")
with_icon_icon_size = _define_token(24)
with_icon_inactive_focus_icon_color = _define_token("md.sys.color.on-surface")
with_icon_inactive_hover_icon_color = _define_token("md.sys.color.on-surface")
with_icon_inactive_icon_color = _define_token("md.sys.color.on-surface-variant")
with_icon_inactive_pressed_icon_color = _define_token("md.sys.color.on-surface")
with_label_text_active_focus_label_text_color = _define_token("md.sys.color.primary")
with_label_text_active_hover_label_text_color = _define_token("md.sys.color.primary")
with_label_text_active_label_text_color = _define_token("md.sys.color.primary")
with_label_text_active_pressed_label_text_color = _define_token("md.sys.color.primary")
with_label_text_inactive_focus_label_text_color = _define_token("md.sys.color.on-surface")
with_label_text_inactive_hover_label_text_color = _define_token("md.sys.color.on-surface")
with_label_text_inactive_label_text_color = _define_token("md.sys.color.on-surface-variant")
with_label_text_inactive_pressed_label_text_color = _define_token("md.sys.color.on-surface")
with_label_text_label_text_font = _define_token("md.sys.typescale.title-small.font")
with_label_text_label_text_line_height = _define_token("md.sys.typescale.title-small.line-height")
with_label_text_label_text_size = _define_token("md.sys.typescale.title-small.size")
with_label_text_label_text_tracking = _define_token("md.sys.typescale.title-small.tracking")
with_label_text_label_text_weight = _define_token("md.sys.typescale.title-small.weight")



from pytest_mock import MockerFixture
from pytestqt.qtbot import QtBot
from qtpy.QtCore import Qt

from material_ui.icon import Icon
from material_ui.menu import Menu, MenuItem


def test_Menu_clicking_item_closes_menu(qtbot: QtBot, mocker: MockerFixture):
    menu = Menu()
    qtbot.addWidget(menu)

    item1 = MenuItem()
    item1.text = "Item"
    item1.setParent(menu)

    menu.open(menu)

    close_menu_spy = mocker.spy(menu, "close_menu")
    qtbot.mouseClick(item1, Qt.MouseButton.LeftButton)
    qtbot.wait_until(lambda: close_menu_spy.call_count == 1, timeout=100)


def test_MenuItem_with_icon(qtbot: QtBot):
    item = MenuItem()
    item.text = "Item"
    icon = Icon()
    icon.icon_name = "check"
    item.leading_icon = icon
    assert item._leading_icon_wrapper.findChild(Icon) is not None
    qtbot.addWidget(item)


def test_MenuItem_with_icon_then_none(qtbot: QtBot):
    item = MenuItem()
    item.text = "Item"
    icon = Icon()
    icon.icon_name = "check"
    item.leading_icon = icon
    assert item._leading_icon_wrapper.findChild(Icon) is not None
    item.leading_icon = None
    assert item._leading_icon_wrapper.findChild(Icon) is None
    qtbot.addWidget(item)



"""Buttons prompt most actions in a UI."""

from material_ui.buttons.elevated_button import ElevatedButton
from material_ui.buttons.filled_button import FilledButton
from material_ui.buttons.filled_tonal_button import FilledTonalButton
from material_ui.buttons.outlined_button import OutlinedButton
from material_ui.buttons.text_button import TextButton


__all__ = [
    "ElevatedButton",
    "FilledButton",
    "FilledTonalButton",
    "OutlinedButton",
    "TextButton",
]



"""Design tokens for md.comp.time-input."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

container_color = _define_token("md.sys.color.surface-container-high")
container_elevation = _define_token("md.sys.elevation.level3")
container_shape = _define_token("md.sys.shape.corner.extra-large")
focus_indicator_color = _define_token("md.sys.color.secondary")
focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
headline_color = _define_token("md.sys.color.on-surface-variant")
headline_font = _define_token("md.sys.typescale.label-medium.font")
headline_line_height = _define_token("md.sys.typescale.label-medium.line-height")
headline_size = _define_token("md.sys.typescale.label-medium.size")
headline_tracking = _define_token("md.sys.typescale.label-medium.tracking")
headline_weight = _define_token("md.sys.typescale.label-medium.weight")
period_selector_container_height = _define_token(72)
period_selector_container_shape = _define_token("md.sys.shape.corner.small")
period_selector_container_width = _define_token(52)
period_selector_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
period_selector_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
period_selector_label_text_font = _define_token("md.sys.typescale.title-medium.font")
period_selector_label_text_line_height = _define_token("md.sys.typescale.title-medium.line-height")
period_selector_label_text_size = _define_token("md.sys.typescale.title-medium.size")
period_selector_label_text_tracking = _define_token("md.sys.typescale.title-medium.tracking")
period_selector_label_text_weight = _define_token("md.sys.typescale.title-medium.weight")
period_selector_outline_color = _define_token("md.sys.color.outline")
period_selector_outline_width = _define_token(1)
period_selector_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
period_selector_selected_container_color = _define_token("md.sys.color.tertiary-container")
period_selector_selected_focus_label_text_color = _define_token("md.sys.color.on-tertiary-container")
period_selector_selected_focus_state_layer_color = _define_token("md.sys.color.on-tertiary-container")
period_selector_selected_hover_label_text_color = _define_token("md.sys.color.on-tertiary-container")
period_selector_selected_hover_state_layer_color = _define_token("md.sys.color.on-tertiary-container")
period_selector_selected_label_text_color = _define_token("md.sys.color.on-tertiary-container")
period_selector_selected_pressed_label_text_color = _define_token("md.sys.color.on-tertiary-container")
period_selector_selected_pressed_state_layer_color = _define_token("md.sys.color.on-tertiary-container")
period_selector_unselected_focus_label_text_color = _define_token("md.sys.color.on-surface-variant")
period_selector_unselected_focus_state_layer_color = _define_token("md.sys.color.on-surface-variant")
period_selector_unselected_hover_label_text_color = _define_token("md.sys.color.on-surface-variant")
period_selector_unselected_hover_state_layer_color = _define_token("md.sys.color.on-surface-variant")
period_selector_unselected_label_text_color = _define_token("md.sys.color.on-surface-variant")
period_selector_unselected_pressed_label_text_color = _define_token("md.sys.color.on-surface-variant")
period_selector_unselected_pressed_state_layer_color = _define_token("md.sys.color.on-surface-variant")
surface_tint_layer_color = _define_token("md.sys.color.surface-tint")
time_input_field_container_color = _define_token("md.sys.color.surface-container-highest")
time_input_field_container_height = _define_token(72)
time_input_field_container_shape = _define_token("md.sys.shape.corner.small")
time_input_field_container_width = _define_token(96)
time_input_field_focus_container_color = _define_token("md.sys.color.primary-container")
time_input_field_focus_label_text_color = _define_token("md.sys.color.on-primary-container")
time_input_field_focus_outline_color = _define_token("md.sys.color.primary")
time_input_field_focus_outline_width = _define_token(2)
time_input_field_hover_label_text_color = _define_token("md.sys.color.on-surface")
time_input_field_hover_state_layer_color = _define_token("md.sys.color.on-surface")
time_input_field_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
time_input_field_label_text_color = _define_token("md.sys.color.on-surface")
time_input_field_label_text_font = _define_token("md.sys.typescale.display-medium.font")
time_input_field_label_text_line_height = _define_token("md.sys.typescale.display-medium.line-height")
time_input_field_label_text_size = _define_token("md.sys.typescale.display-medium.size")
time_input_field_label_text_tracking = _define_token("md.sys.typescale.display-medium.tracking")
time_input_field_label_text_weight = _define_token("md.sys.typescale.display-medium.weight")
time_input_field_separator_color = _define_token("md.sys.color.on-surface")
time_input_field_separator_font = _define_token("md.sys.typescale.display-large.font")
time_input_field_separator_line_height = _define_token("md.sys.typescale.display-large.line-height")
time_input_field_separator_size = _define_token("md.sys.typescale.display-large.size")
time_input_field_separator_tracking = _define_token("md.sys.typescale.display-large.tracking")
time_input_field_separator_weight = _define_token("md.sys.typescale.display-large.weight")
time_input_field_supporting_text_color = _define_token("md.sys.color.on-surface-variant")
time_input_field_supporting_text_font = _define_token("md.sys.typescale.body-small.font")
time_input_field_supporting_text_line_height = _define_token("md.sys.typescale.body-small.line-height")
time_input_field_supporting_text_size = _define_token("md.sys.typescale.body-small.size")
time_input_field_supporting_text_tracking = _define_token("md.sys.typescale.body-small.tracking")
time_input_field_supporting_text_weight = _define_token("md.sys.typescale.body-small.weight")



"""Design tokens for md.comp.checkbox."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

container_height = _define_token(18)
container_shape = _define_token("md.sys.shape.corner.extra-extra-small")
container_size = _define_token(18)
container_width = _define_token(18)
disabled_selected_icon_color = _define_token("md.sys.color.primary")
disabled_selected_icon_opacity = _define_token(0.38)
disabled_unselected_icon_color = _define_token("md.sys.color.primary")
disabled_unselected_icon_opacity = _define_token(0.38)
error_focus_state_layer_color = _define_token("md.sys.color.error")
error_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
error_hover_state_layer_color = _define_token("md.sys.color.error")
error_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
error_pressed_state_layer_color = _define_token("md.sys.color.error")
error_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
focus_indicator_color = _define_token("md.sys.color.secondary")
focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
icon_size = _define_token(18)
selected_container_color = _define_token("md.sys.color.primary")
selected_disabled_container_color = _define_token("md.sys.color.on-surface")
selected_disabled_container_opacity = _define_token(0.38)
selected_disabled_container_outline_width = _define_token(0)
selected_disabled_icon_color = _define_token("md.sys.color.surface")
selected_error_container_color = _define_token("md.sys.color.error")
selected_error_focus_container_color = _define_token("md.sys.color.error")
selected_error_focus_icon_color = _define_token("md.sys.color.on-error")
selected_error_focus_outline_width = _define_token(0)
selected_error_hover_container_color = _define_token("md.sys.color.error")
selected_error_hover_icon_color = _define_token("md.sys.color.on-error")
selected_error_hover_outline_width = _define_token(0)
selected_error_icon_color = _define_token("md.sys.color.on-error")
selected_error_pressed_container_color = _define_token("md.sys.color.error")
selected_error_pressed_icon_color = _define_token("md.sys.color.on-error")
selected_error_pressed_outline_width = _define_token(0)
selected_focus_container_color = _define_token("md.sys.color.primary")
selected_focus_icon_color = _define_token("md.sys.color.on-primary")
selected_focus_outline_width = _define_token(0)
selected_focus_state_layer_color = _define_token("md.sys.color.primary")
selected_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
selected_hover_container_color = _define_token("md.sys.color.primary")
selected_hover_icon_color = _define_token("md.sys.color.on-primary")
selected_hover_outline_width = _define_token(0)
selected_hover_state_layer_color = _define_token("md.sys.color.primary")
selected_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
selected_icon_color = _define_token("md.sys.color.on-primary")
selected_outline_width = _define_token(0)
selected_pressed_container_color = _define_token("md.sys.color.primary")
selected_pressed_icon_color = _define_token("md.sys.color.on-primary")
selected_pressed_outline_width = _define_token(0)
selected_pressed_state_layer_color = _define_token("md.sys.color.on-surface")
selected_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
state_layer_shape = _define_token("md.sys.shape.corner.full")
state_layer_size = _define_token(40)
unselected_disabled_container_opacity = _define_token(0.38)
unselected_disabled_outline_color = _define_token("md.sys.color.on-surface")
unselected_disabled_outline_width = _define_token(2)
unselected_error_focus_outline_color = _define_token("md.sys.color.error")
unselected_error_focus_outline_width = _define_token(2)
unselected_error_hover_outline_color = _define_token("md.sys.color.error")
unselected_error_hover_outline_width = _define_token(2)
unselected_error_outline_color = _define_token("md.sys.color.error")
unselected_error_pressed_outline_color = _define_token("md.sys.color.error")
unselected_error_pressed_outline_width = _define_token(2)
unselected_focus_icon_color = _define_token("md.sys.color.on-surface")
unselected_focus_outline_color = _define_token("md.sys.color.on-surface")
unselected_focus_outline_width = _define_token(2)
unselected_focus_state_layer_color = _define_token("md.sys.color.on-surface")
unselected_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
unselected_hover_icon_color = _define_token("md.sys.color.on-surface")
unselected_hover_outline_color = _define_token("md.sys.color.on-surface")
unselected_hover_outline_width = _define_token(2)
unselected_hover_state_layer_color = _define_token("md.sys.color.on-surface")
unselected_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
unselected_icon_color = _define_token("md.sys.color.on-surface")
unselected_outline_color = _define_token("md.sys.color.on-surface-variant")
unselected_outline_width = _define_token(2)
unselected_pressed_icon_color = _define_token("md.sys.color.on-surface")
unselected_pressed_outline_color = _define_token("md.sys.color.on-surface")
unselected_pressed_outline_width = _define_token(2)
unselected_pressed_state_layer_color = _define_token("md.sys.color.primary")
unselected_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")



"""Design tokens for md.ref.typeface."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

brand = _define_token("Roboto")
plain = _define_token("Roboto")
weight_bold = _define_token(700)
weight_medium = _define_token(500)
weight_regular = _define_token(400)



"""Sample of the typography component."""

from material_ui._component import Component
from material_ui.layout_basics import Stack
from material_ui.tokens import md_sys_color
from material_ui.typography import Typography
from qtpy.QtCore import QMargins
from qtpy.QtWidgets import QApplication


class SampleTypography(Component):
    def __init__(self) -> None:
        super().__init__()

        self.sx = {"background-color": md_sys_color.background}

        stack = Stack(gap=5, margins=QMargins(40, 30, 40, 30))

        for variant in [
            "display-large",
            "display-medium",
            "headline-large",
            "headline-medium",
            "headline-small",
            "title-large",
            "title-medium",
            "title-small",
            "body-large",
            "body-medium",
            "body-small",
            "label-large",
            "label-large-prominent",
            "label-medium",
            "label-medium-prominent",
            "label-small",
        ]:
            typography = Typography()
            typography.variant = variant
            typography.text = variant.replace("-", " ").title()
            stack.add_widget(typography)

        self.overlay_widget(stack)


def main() -> None:
    app = QApplication()
    window = SampleTypography()
    window.show()
    app.exec()


if __name__ == "__main__":
    main()



"""Design tokens for md.comp.badge."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

color = _define_token("md.sys.color.error")
large_color = _define_token("md.sys.color.error")
large_label_text_color = _define_token("md.sys.color.on-error")
large_label_text_font = _define_token("md.sys.typescale.label-small.font")
large_label_text_line_height = _define_token("md.sys.typescale.label-small.line-height")
large_label_text_size = _define_token("md.sys.typescale.label-small.size")
large_label_text_tracking = _define_token("md.sys.typescale.label-small.tracking")
large_label_text_weight = _define_token("md.sys.typescale.label-small.weight")
large_shape = _define_token("md.sys.shape.corner.full")
large_size = _define_token(16)
shape = _define_token("md.sys.shape.corner.full")
size = _define_token(6)



"""Elevated Button."""

from material_ui._component import effect
from material_ui.buttons._button_base import ButtonBase
from material_ui.tokens import md_comp_elevated_button as tokens


class ElevatedButton(ButtonBase):
    """ElevatedButton."""

    def __init__(self) -> None:
        super().__init__()
        self._drop_shadow.shadow_color = tokens.container_shadow_color
        self._drop_shadow.elevation = tokens.container_elevation
        self._container.color = tokens.container_color
        self._ripple.color = tokens.pressed_state_layer_color
        self._label.font_family = tokens.label_text_font
        self._label.font_size = tokens.label_text_size
        self._label.font_weight = tokens.label_text_weight
        self._label.color = tokens.label_text_color

    @effect(ButtonBase.hovered, ButtonBase.pressed)
    def _update_drop_shadow_elevation(self) -> None:
        self._drop_shadow.animate_elevation_to(
            {
                True: tokens.container_elevation,
                self.hovered: tokens.hover_container_elevation,
                self.pressed: tokens.pressed_container_elevation,
            }[True],
        )



"""Circular progress indicator."""

import math
import time
from typing import cast

from qtpy.QtCore import (
    QEasingCurve,
    QMargins,
    QPropertyAnimation,
    QRect,
    QSize,
    QTimerEvent,
)
from qtpy.QtGui import QPainter, QPaintEvent, QPen
from qtpy.QtWidgets import QSizePolicy

from material_ui._component import effect, use_state
from material_ui.progress_indicators._base_progress import BaseProgress
from material_ui.tokens import md_comp_circular_progress_indicator as tokens
from material_ui.tokens._utils import resolve_token


class CircularProgress(BaseProgress):
    """Circular progress indicator."""

    def __init__(self) -> None:
        super().__init__()
        self.setSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
        size = resolve_token(tokens.size)
        self.setFixedSize(QSize(size, size))

        self._timer_id: int | None = None

    _start_angle = use_state(0)
    _span_angle = use_state(0)
    _t = use_state(0.0)

    @effect(BaseProgress.indeterminate)
    def _start_stop_indeterminate_animation(self) -> None:
        """Start or stop the indeterminate animation."""
        if self.indeterminate:
            # Use a short interval to make sure animations are applied.
            # Because we are using update to enqueue the paint, Qt will
            # supposedly optimize the actual paint calls until
            # necessary.
            self._timer_id = self.startTimer(10)
        elif self._timer_id:
            self.killTimer(self._timer_id)

    def timerEvent(self, event: QTimerEvent) -> None:  # noqa: N802
        """Animate the indeterminate progress."""
        if event.timerId() == self._timer_id:
            self._t = time.time_ns() / 1_000_000_000
        else:
            super().timerEvent(event)

    @effect(BaseProgress.value, BaseProgress.indeterminate)
    def _update_draw_parameters(self) -> None:
        if not (0.0 <= self.value <= 1.0):
            raise ValueError
        # Qt angles start at 3 o'clock, go counterclockwise and use
        # 1/16th of a degree as units.
        if not self.indeterminate:
            self._start_angle = 90 * 16
            self._span_angle = -int(self.value * 360 * 16)
        else:
            start_angle_animation = QPropertyAnimation()
            start_angle_animation.setParent(self)
            start_angle_animation.setTargetObject(self._find_state("_start_angle"))
            start_angle_animation.setPropertyName(b"_qt_property")
            start_angle_animation.setStartValue(-45 * 16)
            start_angle_animation.setKeyValueAt(0.45, -105 * 16)
            start_angle_animation.setKeyValueAt(0.65, -105 * 16)
            start_angle_animation.setEndValue(-405 * 16)
            start_angle_animation.setDuration(1333)
            start_angle_animation.setEasingCurve(QEasingCurve.Type.InOutQuad)
            start_angle_animation.setLoopCount(-1)
            start_angle_animation.start()

            span_angle_animation = QPropertyAnimation()
            span_angle_animation.setParent(self)
            span_angle_animation.setTargetObject(self._find_state("_span_angle"))
            span_angle_animation.setPropertyName(b"_qt_property")
            span_angle_animation.setStartValue(-10 * 16)
            span_angle_animation.setKeyValueAt(0.45, -270 * 16)
            span_angle_animation.setKeyValueAt(0.65, -270 * 16)
            span_angle_animation.setEndValue(-10 * 16)
            span_angle_animation.setDuration(1333)
            span_angle_animation.setEasingCurve(QEasingCurve.Type.InOutQuad)
            span_angle_animation.setLoopCount(-1)
            span_angle_animation.start()

    @effect(_start_angle, _span_angle, _t)
    def _update_on_angles_change(self) -> None:
        self.update()

    def paintEvent(self, event: QPaintEvent) -> None:  # noqa: N802
        """Paint the circular progress indicator."""
        super().paintEvent(event)

        painter = QPainter(self)
        painter.setRenderHint(QPainter.RenderHint.Antialiasing)

        color = resolve_token(tokens.active_indicator_color)
        painter.setPen(QPen(color, float(self._thickness)))

        # For indeterminate

        start_angle = self._start_angle + self._indeterminate_angle_offset()
        painter.drawArc(self._arc_rect, start_angle, self._span_angle)

        painter.end()

    @property
    def _thickness(self) -> int:
        """Returns the thickness of the line."""
        return cast("int", resolve_token(tokens.active_indicator_width))

    @property
    def _arc_rect(self) -> QRect:
        """Returns the rectangle for the arc."""
        # Subtract padding so the line isn't drawn half out of bounds.
        p = self._thickness + 1
        return self.rect().marginsRemoved(QMargins(p, p, p, p))

    def _indeterminate_angle_offset(self) -> int:
        if not self.indeterminate:
            return 0

        layer1 = (self._t * 0.34) % 1
        layer2 = (math.sin(self._t * math.pi * 3) + 1) * 0.024
        return int((layer1 + layer2) * 360 * -16)



"""Design tokens for md.comp.text-button."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

container_elevation = _define_token("md.sys.elevation.level0")
container_height = _define_token(40)
container_shape = _define_token("md.sys.shape.corner.full")
disabled_label_text_color = _define_token("md.sys.color.on-surface")
disabled_label_text_opacity = _define_token(0.38)
focus_indicator_color = _define_token("md.sys.color.secondary")
focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
focus_label_text_color = _define_token("md.sys.color.primary")
focus_state_layer_color = _define_token("md.sys.color.primary")
focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
hover_label_text_color = _define_token("md.sys.color.primary")
hover_state_layer_color = _define_token("md.sys.color.primary")
hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
label_text_color = _define_token("md.sys.color.primary")
label_text_font = _define_token("md.sys.typescale.label-large.font")
label_text_line_height = _define_token("md.sys.typescale.label-large.line-height")
label_text_size = _define_token("md.sys.typescale.label-large.size")
label_text_tracking = _define_token("md.sys.typescale.label-large.tracking")
label_text_weight = _define_token("md.sys.typescale.label-large.weight")
pressed_label_text_color = _define_token("md.sys.color.primary")
pressed_state_layer_color = _define_token("md.sys.color.primary")
pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
with_icon_disabled_icon_color = _define_token("md.sys.color.on-surface")
with_icon_disabled_icon_opacity = _define_token(0.38)
with_icon_focus_icon_color = _define_token("md.sys.color.primary")
with_icon_hover_icon_color = _define_token("md.sys.color.primary")
with_icon_icon_color = _define_token("md.sys.color.primary")
with_icon_icon_size = _define_token(18)
with_icon_pressed_icon_color = _define_token("md.sys.color.primary")



"""Design tokens for md.comp.fab."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

branded_container_color = _define_token("md.sys.color.surface-container-high")
branded_container_elevation = _define_token("md.sys.elevation.level3")
branded_container_height = _define_token(56)
branded_container_shadow_color = _define_token("md.sys.color.shadow")
branded_container_shape = _define_token("md.sys.shape.corner.large")
branded_container_surface_tint_layer_color = _define_token("md.sys.color.surface-tint")
branded_container_width = _define_token(56)
branded_focus_container_elevation = _define_token("md.sys.elevation.level3")
branded_focus_indicator_color = _define_token("md.sys.color.secondary")
branded_focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
branded_focus_state_layer_color = _define_token("md.sys.color.primary")
branded_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
branded_hover_container_elevation = _define_token("md.sys.elevation.level4")
branded_hover_state_layer_color = _define_token("md.sys.color.primary")
branded_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
branded_icon_size = _define_token(36)
branded_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
branded_large_container_color = _define_token("md.sys.color.surface-container-high")
branded_large_container_elevation = _define_token("md.sys.elevation.level3")
branded_large_container_height = _define_token(96)
branded_large_container_shadow_color = _define_token("md.sys.color.shadow")
branded_large_container_shape = _define_token("md.sys.shape.corner.extra-large")
branded_large_container_surface_tint_layer_color = _define_token("md.sys.color.surface-tint")
branded_large_container_width = _define_token(96)
branded_large_focus_container_elevation = _define_token("md.sys.elevation.level3")
branded_large_focus_indicator_color = _define_token("md.sys.color.secondary")
branded_large_focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
branded_large_focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
branded_large_focus_state_layer_color = _define_token("md.sys.color.primary")
branded_large_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
branded_large_hover_container_elevation = _define_token("md.sys.elevation.level4")
branded_large_hover_state_layer_color = _define_token("md.sys.color.primary")
branded_large_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
branded_large_icon_color = _define_token("md.sys.color.on-secondary")
branded_large_icon_size = _define_token(48)
branded_large_lowered_container_color = _define_token("md.sys.color.surface-container-low")
branded_large_lowered_container_elevation = _define_token("md.sys.elevation.level1")
branded_large_lowered_focus_container_elevation = _define_token("md.sys.elevation.level1")
branded_large_lowered_hover_container_elevation = _define_token("md.sys.elevation.level2")
branded_large_lowered_pressed_container_elevation = _define_token("md.sys.elevation.level1")
branded_large_pressed_container_elevation = _define_token("md.sys.elevation.level3")
branded_large_pressed_state_layer_color = _define_token("md.sys.color.primary")
branded_large_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
branded_lowered_container_color = _define_token("md.sys.color.surface-container-low")
branded_lowered_container_elevation = _define_token("md.sys.elevation.level1")
branded_lowered_focus_container_elevation = _define_token("md.sys.elevation.level1")
branded_lowered_hover_container_elevation = _define_token("md.sys.elevation.level2")
branded_lowered_pressed_container_elevation = _define_token("md.sys.elevation.level1")
branded_pressed_container_elevation = _define_token("md.sys.elevation.level3")
branded_pressed_state_layer_color = _define_token("md.sys.color.primary")
branded_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
primary_container_color = _define_token("md.sys.color.primary-container")
primary_container_elevation = _define_token("md.sys.elevation.level3")
primary_container_height = _define_token(56)
primary_container_shadow_color = _define_token("md.sys.color.shadow")
primary_container_shape = _define_token("md.sys.shape.corner.large")
primary_container_width = _define_token(56)
primary_focus_container_elevation = _define_token("md.sys.elevation.level3")
primary_focus_icon_color = _define_token("md.sys.color.on-primary-container")
primary_focus_indicator_color = _define_token("md.sys.color.secondary")
primary_focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
primary_focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
primary_focus_state_layer_color = _define_token("md.sys.color.on-primary-container")
primary_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
primary_hover_container_elevation = _define_token("md.sys.elevation.level4")
primary_hover_icon_color = _define_token("md.sys.color.on-primary-container")
primary_hover_state_layer_color = _define_token("md.sys.color.on-primary-container")
primary_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
primary_icon_color = _define_token("md.sys.color.on-primary-container")
primary_icon_size = _define_token(24)
primary_large_container_color = _define_token("md.sys.color.primary-container")
primary_large_container_elevation = _define_token("md.sys.elevation.level3")
primary_large_container_height = _define_token(96)
primary_large_container_shadow_color = _define_token("md.sys.color.shadow")
primary_large_container_shape = _define_token("md.sys.shape.corner.extra-large")
primary_large_container_width = _define_token(96)
primary_large_focus_container_elevation = _define_token("md.sys.elevation.level3")
primary_large_focus_icon_color = _define_token("md.sys.color.on-primary-container")
primary_large_focus_indicator_color = _define_token("md.sys.color.secondary")
primary_large_focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
primary_large_focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
primary_large_focus_state_layer_color = _define_token("md.sys.color.on-primary-container")
primary_large_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
primary_large_hover_container_elevation = _define_token("md.sys.elevation.level4")
primary_large_hover_icon_color = _define_token("md.sys.color.on-primary-container")
primary_large_hover_state_layer_color = _define_token("md.sys.color.on-primary-container")
primary_large_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
primary_large_icon_color = _define_token("md.sys.color.on-primary-container")
primary_large_icon_size = _define_token(36)
primary_large_lowered_container_elevation = _define_token("md.sys.elevation.level1")
primary_large_lowered_focus_container_elevation = _define_token("md.sys.elevation.level1")
primary_large_lowered_hover_container_elevation = _define_token("md.sys.elevation.level2")
primary_large_lowered_pressed_container_elevation = _define_token("md.sys.elevation.level1")
primary_large_pressed_container_elevation = _define_token("md.sys.elevation.level3")
primary_large_pressed_icon_color = _define_token("md.sys.color.on-primary-container")
primary_large_pressed_state_layer_color = _define_token("md.sys.color.on-primary-container")
primary_large_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
primary_lowered_container_elevation = _define_token("md.sys.elevation.level1")
primary_lowered_focus_container_elevation = _define_token("md.sys.elevation.level1")
primary_lowered_hover_container_elevation = _define_token("md.sys.elevation.level2")
primary_lowered_pressed_container_elevation = _define_token("md.sys.elevation.level1")
primary_pressed_container_elevation = _define_token("md.sys.elevation.level3")
primary_pressed_icon_color = _define_token("md.sys.color.on-primary-container")
primary_pressed_state_layer_color = _define_token("md.sys.color.on-primary-container")
primary_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
primary_small_container_color = _define_token("md.sys.color.primary-container")
primary_small_container_elevation = _define_token("md.sys.elevation.level3")
primary_small_container_height = _define_token(40)
primary_small_container_shadow_color = _define_token("md.sys.color.shadow")
primary_small_container_shape = _define_token("md.sys.shape.corner.medium")
primary_small_container_width = _define_token(40)
primary_small_focus_container_elevation = _define_token("md.sys.elevation.level3")
primary_small_focus_icon_color = _define_token("md.sys.color.on-primary-container")
primary_small_focus_indicator_color = _define_token("md.sys.color.secondary")
primary_small_focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
primary_small_focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
primary_small_focus_state_layer_color = _define_token("md.sys.color.on-primary-container")
primary_small_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
primary_small_hover_container_elevation = _define_token("md.sys.elevation.level4")
primary_small_hover_icon_color = _define_token("md.sys.color.on-primary-container")
primary_small_hover_state_layer_color = _define_token("md.sys.color.on-primary-container")
primary_small_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
primary_small_icon_color = _define_token("md.sys.color.on-primary-container")
primary_small_icon_size = _define_token(24)
primary_small_lowered_container_elevation = _define_token("md.sys.elevation.level1")
primary_small_lowered_focus_container_elevation = _define_token("md.sys.elevation.level1")
primary_small_lowered_hover_container_elevation = _define_token("md.sys.elevation.level2")
primary_small_lowered_pressed_container_elevation = _define_token("md.sys.elevation.level1")
primary_small_pressed_container_elevation = _define_token("md.sys.elevation.level3")
primary_small_pressed_icon_color = _define_token("md.sys.color.on-primary-container")
primary_small_pressed_state_layer_color = _define_token("md.sys.color.on-primary-container")
primary_small_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
secondary_container_color = _define_token("md.sys.color.secondary-container")
secondary_container_elevation = _define_token("md.sys.elevation.level3")
secondary_container_height = _define_token(56)
secondary_container_shadow_color = _define_token("md.sys.color.shadow")
secondary_container_shape = _define_token("md.sys.shape.corner.large")
secondary_container_width = _define_token(56)
secondary_focus_container_elevation = _define_token("md.sys.elevation.level3")
secondary_focus_icon_color = _define_token("md.sys.color.on-secondary-container")
secondary_focus_indicator_color = _define_token("md.sys.color.secondary")
secondary_focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
secondary_focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
secondary_focus_state_layer_color = _define_token("md.sys.color.on-secondary-container")
secondary_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
secondary_hover_container_elevation = _define_token("md.sys.elevation.level4")
secondary_hover_icon_color = _define_token("md.sys.color.on-secondary-container")
secondary_hover_state_layer_color = _define_token("md.sys.color.on-secondary-container")
secondary_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
secondary_icon_color = _define_token("md.sys.color.on-secondary-container")
secondary_icon_size = _define_token(24)
secondary_large_container_color = _define_token("md.sys.color.secondary-container")
secondary_large_container_elevation = _define_token("md.sys.elevation.level3")
secondary_large_container_height = _define_token(96)
secondary_large_container_shadow_color = _define_token("md.sys.color.shadow")
secondary_large_container_shape = _define_token("md.sys.shape.corner.extra-large")
secondary_large_container_width = _define_token(96)
secondary_large_focus_container_elevation = _define_token("md.sys.elevation.level3")
secondary_large_focus_icon_color = _define_token("md.sys.color.on-secondary-container")
secondary_large_focus_indicator_color = _define_token("md.sys.color.secondary")
secondary_large_focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
secondary_large_focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
secondary_large_focus_state_layer_color = _define_token("md.sys.color.on-secondary-container")
secondary_large_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
secondary_large_hover_container_elevation = _define_token("md.sys.elevation.level4")
secondary_large_hover_icon_color = _define_token("md.sys.color.on-secondary-container")
secondary_large_hover_state_layer_color = _define_token("md.sys.color.on-secondary-container")
secondary_large_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
secondary_large_icon_color = _define_token("md.sys.color.on-secondary-container")
secondary_large_icon_size = _define_token(36)
secondary_large_lowered_container_elevation = _define_token("md.sys.elevation.level1")
secondary_large_lowered_focus_container_elevation = _define_token("md.sys.elevation.level1")
secondary_large_lowered_hover_container_elevation = _define_token("md.sys.elevation.level2")
secondary_large_lowered_pressed_container_elevation = _define_token("md.sys.elevation.level1")
secondary_large_pressed_container_elevation = _define_token("md.sys.elevation.level3")
secondary_large_pressed_icon_color = _define_token("md.sys.color.on-secondary-container")
secondary_large_pressed_state_layer_color = _define_token("md.sys.color.on-secondary-container")
secondary_large_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
secondary_lowered_container_elevation = _define_token("md.sys.elevation.level1")
secondary_lowered_focus_container_elevation = _define_token("md.sys.elevation.level1")
secondary_lowered_hover_container_elevation = _define_token("md.sys.elevation.level2")
secondary_lowered_pressed_container_elevation = _define_token("md.sys.elevation.level1")
secondary_pressed_container_elevation = _define_token("md.sys.elevation.level3")
secondary_pressed_icon_color = _define_token("md.sys.color.on-secondary-container")
secondary_pressed_state_layer_color = _define_token("md.sys.color.on-secondary-container")
secondary_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
secondary_small_container_color = _define_token("md.sys.color.secondary-container")
secondary_small_container_elevation = _define_token("md.sys.elevation.level3")
secondary_small_container_height = _define_token(40)
secondary_small_container_shadow_color = _define_token("md.sys.color.shadow")
secondary_small_container_shape = _define_token("md.sys.shape.corner.medium")
secondary_small_container_width = _define_token(40)
secondary_small_focus_container_elevation = _define_token("md.sys.elevation.level3")
secondary_small_focus_icon_color = _define_token("md.sys.color.on-secondary-container")
secondary_small_focus_indicator_color = _define_token("md.sys.color.secondary")
secondary_small_focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
secondary_small_focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
secondary_small_focus_state_layer_color = _define_token("md.sys.color.on-secondary-container")
secondary_small_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
secondary_small_hover_container_elevation = _define_token("md.sys.elevation.level4")
secondary_small_hover_icon_color = _define_token("md.sys.color.on-secondary-container")
secondary_small_hover_state_layer_color = _define_token("md.sys.color.on-secondary-container")
secondary_small_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
secondary_small_icon_color = _define_token("md.sys.color.on-secondary-container")
secondary_small_icon_size = _define_token(24)
secondary_small_lowered_container_elevation = _define_token("md.sys.elevation.level1")
secondary_small_lowered_focus_container_elevation = _define_token("md.sys.elevation.level1")
secondary_small_lowered_hover_container_elevation = _define_token("md.sys.elevation.level2")
secondary_small_lowered_pressed_container_elevation = _define_token("md.sys.elevation.level1")
secondary_small_pressed_container_elevation = _define_token("md.sys.elevation.level3")
secondary_small_pressed_icon_color = _define_token("md.sys.color.on-secondary-container")
secondary_small_pressed_state_layer_color = _define_token("md.sys.color.on-secondary-container")
secondary_small_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
surface_container_color = _define_token("md.sys.color.surface-container-high")
surface_container_elevation = _define_token("md.sys.elevation.level3")
surface_container_height = _define_token(56)
surface_container_shadow_color = _define_token("md.sys.color.shadow")
surface_container_shape = _define_token("md.sys.shape.corner.large")
surface_container_surface_tint_layer_color = _define_token("md.sys.color.surface-tint")
surface_container_width = _define_token(56)
surface_focus_container_elevation = _define_token("md.sys.elevation.level3")
surface_focus_icon_color = _define_token("md.sys.color.primary")
surface_focus_indicator_color = _define_token("md.sys.color.secondary")
surface_focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
surface_focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
surface_focus_state_layer_color = _define_token("md.sys.color.primary")
surface_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
surface_hover_container_elevation = _define_token("md.sys.elevation.level4")
surface_hover_icon_color = _define_token("md.sys.color.primary")
surface_hover_state_layer_color = _define_token("md.sys.color.primary")
surface_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
surface_icon_color = _define_token("md.sys.color.primary")
surface_icon_size = _define_token(24)
surface_large_container_color = _define_token("md.sys.color.surface-container-high")
surface_large_container_elevation = _define_token("md.sys.elevation.level3")
surface_large_container_height = _define_token(96)
surface_large_container_shadow_color = _define_token("md.sys.color.shadow")
surface_large_container_shape = _define_token("md.sys.shape.corner.extra-large")
surface_large_container_surface_tint_layer_color = _define_token("md.sys.color.surface-tint")
surface_large_container_width = _define_token(96)
surface_large_focus_container_elevation = _define_token("md.sys.elevation.level3")
surface_large_focus_icon_color = _define_token("md.sys.color.primary")
surface_large_focus_indicator_color = _define_token("md.sys.color.secondary")
surface_large_focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
surface_large_focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
surface_large_focus_state_layer_color = _define_token("md.sys.color.primary")
surface_large_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
surface_large_hover_container_elevation = _define_token("md.sys.elevation.level4")
surface_large_hover_icon_color = _define_token("md.sys.color.primary")
surface_large_hover_state_layer_color = _define_token("md.sys.color.primary")
surface_large_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
surface_large_icon_color = _define_token("md.sys.color.primary")
surface_large_icon_size = _define_token(36)
surface_large_lowered_container_color = _define_token("md.sys.color.surface-container-low")
surface_large_lowered_container_elevation = _define_token("md.sys.elevation.level1")
surface_large_lowered_focus_container_elevation = _define_token("md.sys.elevation.level1")
surface_large_lowered_hover_container_elevation = _define_token("md.sys.elevation.level2")
surface_large_lowered_pressed_container_elevation = _define_token("md.sys.elevation.level1")
surface_large_pressed_container_elevation = _define_token("md.sys.elevation.level3")
surface_large_pressed_icon_color = _define_token("md.sys.color.primary")
surface_large_pressed_state_layer_color = _define_token("md.sys.color.primary")
surface_large_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
surface_lowered_container_color = _define_token("md.sys.color.surface-container-low")
surface_lowered_container_elevation = _define_token("md.sys.elevation.level1")
surface_lowered_focus_container_elevation = _define_token("md.sys.elevation.level1")
surface_lowered_hover_container_elevation = _define_token("md.sys.elevation.level2")
surface_lowered_pressed_container_elevation = _define_token("md.sys.elevation.level1")
surface_pressed_container_elevation = _define_token("md.sys.elevation.level3")
surface_pressed_icon_color = _define_token("md.sys.color.primary")
surface_pressed_state_layer_color = _define_token("md.sys.color.primary")
surface_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
surface_small_container_color = _define_token("md.sys.color.surface-container-high")
surface_small_container_elevation = _define_token("md.sys.elevation.level3")
surface_small_container_height = _define_token(40)
surface_small_container_shadow_color = _define_token("md.sys.color.shadow")
surface_small_container_shape = _define_token("md.sys.shape.corner.medium")
surface_small_container_surface_tint_layer_color = _define_token("md.sys.color.surface-tint")
surface_small_container_width = _define_token(40)
surface_small_focus_container_elevation = _define_token("md.sys.elevation.level3")
surface_small_focus_icon_color = _define_token("md.sys.color.primary")
surface_small_focus_indicator_color = _define_token("md.sys.color.secondary")
surface_small_focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
surface_small_focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
surface_small_focus_state_layer_color = _define_token("md.sys.color.primary")
surface_small_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
surface_small_hover_container_elevation = _define_token("md.sys.elevation.level4")
surface_small_hover_icon_color = _define_token("md.sys.color.primary")
surface_small_hover_state_layer_color = _define_token("md.sys.color.primary")
surface_small_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
surface_small_icon_color = _define_token("md.sys.color.primary")
surface_small_icon_size = _define_token(24)
surface_small_lowered_container_color = _define_token("md.sys.color.surface-container-low")
surface_small_lowered_container_elevation = _define_token("md.sys.elevation.level1")
surface_small_lowered_focus_container_elevation = _define_token("md.sys.elevation.level1")
surface_small_lowered_hover_container_elevation = _define_token("md.sys.elevation.level2")
surface_small_lowered_pressed_container_elevation = _define_token("md.sys.elevation.level1")
surface_small_pressed_container_elevation = _define_token("md.sys.elevation.level3")
surface_small_pressed_icon_color = _define_token("md.sys.color.primary")
surface_small_pressed_state_layer_color = _define_token("md.sys.color.primary")
surface_small_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
tertiary_container_color = _define_token("md.sys.color.tertiary-container")
tertiary_container_elevation = _define_token("md.sys.elevation.level3")
tertiary_container_height = _define_token(56)
tertiary_container_shadow_color = _define_token("md.sys.color.shadow")
tertiary_container_shape = _define_token("md.sys.shape.corner.large")
tertiary_container_width = _define_token(56)
tertiary_focus_container_elevation = _define_token("md.sys.elevation.level3")
tertiary_focus_icon_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_focus_indicator_color = _define_token("md.sys.color.secondary")
tertiary_focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
tertiary_focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
tertiary_focus_state_layer_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
tertiary_hover_container_elevation = _define_token("md.sys.elevation.level4")
tertiary_hover_icon_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_hover_state_layer_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
tertiary_icon_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_icon_size = _define_token(24)
tertiary_large_container_color = _define_token("md.sys.color.tertiary-container")
tertiary_large_container_elevation = _define_token("md.sys.elevation.level3")
tertiary_large_container_height = _define_token(96)
tertiary_large_container_shadow_color = _define_token("md.sys.color.shadow")
tertiary_large_container_shape = _define_token("md.sys.shape.corner.extra-large")
tertiary_large_container_width = _define_token(96)
tertiary_large_focus_container_elevation = _define_token("md.sys.elevation.level3")
tertiary_large_focus_icon_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_large_focus_indicator_color = _define_token("md.sys.color.secondary")
tertiary_large_focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
tertiary_large_focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
tertiary_large_focus_state_layer_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_large_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
tertiary_large_hover_container_elevation = _define_token("md.sys.elevation.level4")
tertiary_large_hover_icon_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_large_hover_state_layer_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_large_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
tertiary_large_icon_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_large_icon_size = _define_token(36)
tertiary_large_lowered_container_elevation = _define_token("md.sys.elevation.level1")
tertiary_large_lowered_focus_container_elevation = _define_token("md.sys.elevation.level1")
tertiary_large_lowered_hover_container_elevation = _define_token("md.sys.elevation.level2")
tertiary_large_lowered_pressed_container_elevation = _define_token("md.sys.elevation.level1")
tertiary_large_pressed_container_elevation = _define_token("md.sys.elevation.level3")
tertiary_large_pressed_icon_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_large_pressed_state_layer_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_large_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
tertiary_lowered_container_elevation = _define_token("md.sys.elevation.level1")
tertiary_lowered_focus_container_elevation = _define_token("md.sys.elevation.level1")
tertiary_lowered_hover_container_elevation = _define_token("md.sys.elevation.level2")
tertiary_lowered_pressed_container_elevation = _define_token("md.sys.elevation.level1")
tertiary_pressed_container_elevation = _define_token("md.sys.elevation.level3")
tertiary_pressed_icon_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_pressed_state_layer_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
tertiary_small_container_color = _define_token("md.sys.color.tertiary-container")
tertiary_small_container_elevation = _define_token("md.sys.elevation.level3")
tertiary_small_container_height = _define_token(40)
tertiary_small_container_shadow_color = _define_token("md.sys.color.shadow")
tertiary_small_container_shape = _define_token("md.sys.shape.corner.medium")
tertiary_small_container_width = _define_token(40)
tertiary_small_focus_container_elevation = _define_token("md.sys.elevation.level3")
tertiary_small_focus_icon_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_small_focus_indicator_color = _define_token("md.sys.color.secondary")
tertiary_small_focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.outer-offset")
tertiary_small_focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
tertiary_small_focus_state_layer_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_small_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
tertiary_small_hover_container_elevation = _define_token("md.sys.elevation.level4")
tertiary_small_hover_icon_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_small_hover_state_layer_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_small_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
tertiary_small_icon_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_small_icon_size = _define_token(24)
tertiary_small_lowered_container_elevation = _define_token("md.sys.elevation.level1")
tertiary_small_lowered_focus_container_elevation = _define_token("md.sys.elevation.level1")
tertiary_small_lowered_hover_container_elevation = _define_token("md.sys.elevation.level2")
tertiary_small_lowered_pressed_container_elevation = _define_token("md.sys.elevation.level1")
tertiary_small_pressed_container_elevation = _define_token("md.sys.elevation.level3")
tertiary_small_pressed_icon_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_small_pressed_state_layer_color = _define_token("md.sys.color.on-tertiary-container")
tertiary_small_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")



from collections import defaultdict
from dataclasses import dataclass, replace
from functools import partial
from itertools import batched, product
import json
from pathlib import Path

import re
import tempfile
from typing import Callable
import httpx
import asyncio
from qtpy.QtGui import QColor

from material_ui.tokens._utils import TokenValue, to_python_name


TOKEN_TABLE_URL_FORMAT = (
    "https://m3.material.io/_dsm/data/dsdb-m3/latest/TOKEN_TABLE.json"
)


async def fetch_token_tables(no_cache: bool = False) -> list[dict]:
    """Fetch the token tables from the Material API.

    Args:
        no_cache: Results are cached to disk. If no_cache is True, the
            cache is cleared first.

    Returns:
        A list of dictionaries containing the token tables.
    """
    cache_path = Path(tempfile.gettempdir()) / "c28a72b4-37c2-448b-9bbf-c143a4186ffb"
    if no_cache:
        cache_path.unlink(missing_ok=True)
    if cache_path.exists():
        with open(cache_path) as f:
            return json.load(f)
    token_table_ids = list(map("".join, batched(input("TOKEN_TABLE_IDS"), 16)))
    async with httpx.AsyncClient() as client:
        url_fn = TOKEN_TABLE_URL_FORMAT.replace("json", "{}.json").format
        responses = await asyncio.gather(*map(client.get, map(url_fn, token_table_ids)))
    ret_val = [x.json() for x in responses]
    with open(cache_path, "w") as f:
        json.dump(ret_val, f)
    return ret_val


DEFAULT_CONTEXT_TERMS = {"light", "3p", "dynamic"}


def get_tree_context_score(token_table: dict, tree: dict, terms: set[str]) -> float:
    """Returns score of a tree based on context tags.

    Raises:
        RuntimeError: The tree does not have a context defined.
    """
    if "contextTags" not in tree:
        raise RuntimeError("Tree does not have a context defined")
    resolved_tags = map(partial(resolve_context_tag, token_table), tree["contextTags"])
    tree_tag_names = {tag["tagName"] for tag in resolved_tags}
    difference = tree_tag_names.difference(terms)
    return len(terms) - len(difference)


def resolve_context_tag(token_table: dict, name: str) -> dict | None:
    return next(
        (
            context_tag
            for context_tag in token_table["system"]["tags"]
            if context_tag["name"] == name
        ),
        None,
    )


def find_matching_ref_tree(
    token_table: dict, trees: list[dict], terms: set[str]
) -> dict:
    """Find the contextual reference tree matching a given context.

    Args:
        trees: List of contextual reference trees.
        terms: Context tag names to match on.

    Returns:
        Highest scoring tree based on tags. If no trees define a
        context, the first one is returned.
    """
    if trees and all("contextTags" not in x for x in trees):
        return trees[0]  # no contexts defined - return first one
    score_fn = partial(get_tree_context_score, token_table, terms=terms)
    return next(iter(sorted(trees, key=score_fn, reverse=True)), None)


@dataclass
class ParsedToken:
    """Token that was parsed."""

    name: str
    value: TokenValue


ParsedTokens = list[ParsedToken]


def merge_parsed_tokens(list_a: ParsedTokens, list_b: ParsedTokens) -> ParsedTokens:
    """Update the first list with the second one."""
    list_b_token_names = {x.name for x in list_b}
    return list_b + [x for x in list_a if x.name not in list_b_token_names]


def parse_tokens(
    token_tables: list[dict], context_terms: set[str] | None = None
) -> ParsedTokens:
    """Parse the token tables into a merged list of tokens.

    Args:
        token_tables: The token tables to parse.
        context_terms: The context terms to use. If None, all contexts
            are merged together, ending with the default context.

    Returns:
        A list of parsed tokens.
    """
    ret_val: ParsedTokens = []

    # Recursive entry point to merge all contexts.
    if context_terms is None:
        # Collect all context terms from the token tables. Different
        # contexts will get combined with common keys replaced by the
        # last one.
        for context_terms in product(
            ["light", "dark"], ["3p", "1p"], ["dynamic", "non-dynamic"]
        ):
            ret_val = merge_parsed_tokens(
                ret_val, parse_tokens(token_tables, set(context_terms))
            )
        ret_val = merge_parsed_tokens(
            ret_val, parse_tokens(token_tables, DEFAULT_CONTEXT_TERMS)
        )
        return ret_val

    # A specific context.
    for token_table in token_tables:
        ret_val = merge_parsed_tokens(
            ret_val, parse_tokens_in_table(token_table, context_terms)
        )
    return ret_val


def parse_tokens_in_table(token_table: dict, context_terms: set[str]) -> ParsedTokens:
    ret_val: ParsedTokens = []
    tokens = token_table["system"]["tokens"]
    ref_trees = token_table["system"]["contextualReferenceTrees"]
    for token in tokens:
        name = token["name"]
        if name not in ref_trees:
            # Some tokens don't seem to have a reference tree.
            continue
        named_ref_tree = ref_trees[name]["contextualReferenceTree"]
        ref_tree = find_matching_ref_tree(token_table, named_ref_tree, context_terms)[
            "referenceTree"
        ]
        token_name = token["tokenName"]
        # Traverse the recursive tree structure to build a flat list.
        while True:
            reference_value = resolve_value(token_table, ref_tree)
            if reference_value is None:
                # Unable to resolve value - skip.
                break
            token_value = parse_token_value(reference_value)
            if token_value is None:
                # Reached an unsupported value - skip.
                break
            if any(x.name == token_name for x in ret_val):
                # Token already added - skip.
                # If an indirection, children will be the same too.
                break
            ret_val.append(ParsedToken(name=token_name, value=token_value))
            if "childNodes" in ref_tree:
                # Next iteration to go deeper.
                ref_tree = ref_tree["childNodes"][0]
                token_name = token_value
            else:
                # End of the chain.
                break
    return ret_val


def resolve_value(token_table: dict, reference_tree: dict) -> dict | None:
    """Resolve a value"""
    values = token_table["system"]["values"]
    return next(
        (x for x in values if x["name"] == reference_tree["value"]["name"]),
        None,
    )


def parse_token_value(value: dict) -> TokenValue | None:
    """Parse a token value from the token table.

    Args:
        value: The token value.

    Returns:
        The token value if a supported type, otherwise None.

    Raises:
        RuntimeError: Unexpected type of token.
    """

    def without_unit(unit_name: str) -> Callable[[dict], TokenValue]:
        def inner(value: dict) -> TokenValue:
            assert value["unit"] == unit_name
            return value.get("value", 0)

        return inner

    supported_values_transformations = {
        "tokenName": str,
        "color": lambda x: QColor.fromRgbF(
            x.get("red", 0.0),
            x.get("green", 0.0),
            x.get("blue", 0.0),
            x.get("alpha", 1.0),
        ),
        "length": lambda x: {
            # DIPS as int, PERCENT as float
            "PERCENT": x.get("value", 0) / 100,
            "DIPS": x.get("value", 0),
        }[x["unit"]],
        "opacity": float,
        "shape": lambda x: x["family"],
        "fontWeight": int,
        "lineHeight": without_unit("POINTS"),
        "fontTracking": without_unit("POINTS"),
        "fontSize": without_unit("POINTS"),
        "type": lambda _: None,  # unsupported type
        "fontNames": lambda x: x["values"][0],
        "elevation": without_unit("DIPS"),
    }
    for key, transform_fn in supported_values_transformations.items():
        inner_value = value.get(key)
        if inner_value is not None:
            return transform_fn(inner_value)
    raise RuntimeError("unexpected reference value", value)


def group_tokens_by_output_files(tokens: ParsedTokens) -> dict[str, ParsedTokens]:
    """Get the groups by first 3 parts from the tokens.

    Args:
        tokens: The parsed tokens.

    Returns:
        A dictionary mapping component group names to lists of token names.
    """
    ret_val = defaultdict(list)
    for token in tokens:
        match = re.search(r"^(md\.(comp|sys|ref)\..+?)\.", token.name)
        if match:
            group_name = match.group(1)
            ret_val[group_name].append(token)
    return ret_val


TOKENS_OUT_PATH = Path(__file__).parent


def to_var_line(token: ParsedToken) -> str:
    """Code generation for the token."""
    if isinstance(token.value, QColor):
        value = re.search("(QColor.*)$", repr(token.value))[1]
    else:
        value = repr(token.value)
    return f"{to_python_name(token.name)} = _define_token({value})\n"


def generate_py_files(tokens: ParsedTokens) -> None:
    """Generate the Python files for the tokens."""
    component_groups = group_tokens_by_output_files(tokens)
    for group_name, tokens in component_groups.items():
        if group_name == "md.ref.palette":
            # Skip this as the parsing leaves incomplete tokens. Better
            # to write this file manually.
            continue
        with open(TOKENS_OUT_PATH / f"{to_python_name(group_name)}.py", "w") as f:
            f.write(
                f'"""Design tokens for {group_name}."""\n'
                f"\n"
                f"# Auto generated by {Path(__file__).name}\n"
                f"# Do not edit this file directly.\n"
                f"\n"
                f"from material_ui.tokens._utils import define_token as _define_token\n"
                f"\n"
                f"\n"
            )
            for token in tokens:
                # Strip the group name as the file name already has the 'group'.
                token = replace(token, name=token.name[len(group_name) + 1 :])
                f.write(to_var_line(token))


def main() -> None:
    token_tables = asyncio.run(fetch_token_tables())
    tokens = parse_tokens(token_tables)
    # Sort so the files look neater.
    tokens = sorted(tokens, key=lambda x: x.name)
    generate_py_files(tokens)


if __name__ == "__main__":
    main()



"""Outlined Button component."""

from material_ui.buttons._button_base import ButtonBase
from material_ui.tokens import md_comp_outlined_button as tokens


class OutlinedButton(ButtonBase):
    """OutlinedButton."""

    def __init__(self) -> None:
        super().__init__()
        self._container.sx = {
            **self._container.sx,
            "border-color": tokens.outline_color,
            "border-width": tokens.outline_width,
            "border-style": "solid",
        }
        self._container.opacity = 0.0
        self._ripple.color = tokens.pressed_state_layer_color
        self._label.font_family = tokens.label_text_font
        self._label.font_size = tokens.label_text_size
        self._label.font_weight = tokens.label_text_weight
        self._label.color = tokens.label_text_color



from qtpy.QtGui import QColor
import pytest
from material_ui._utils import _stringify_sx_value


@pytest.mark.parametrize(
    ("value", "expected"),
    [
        (QColor(255, 0, 0, 255), "rgba(255,0,0,255)"),
        (QColor("#dad666"), "rgba(218,214,102,255)"),
        (QColor.fromRgbF(0.5, 0.5, 0.25, 0.72), "rgba(128,128,64,184)"),
    ],
)
def test_stringify_sx_value_qcolor(value: QColor, expected: str) -> None:
    assert _stringify_sx_value(value) == expected


def test_stringify_sx_value_string() -> None:
    assert _stringify_sx_value("30px") == "30px"
    assert _stringify_sx_value("1px solid red") == "1px solid red"



"""Design tokens for md.comp.divider."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

color = _define_token("md.sys.color.outline-variant")
thickness = _define_token(1)



"""Design tokens for md.sys.shape."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

corner_extra_large = _define_token("SHAPE_FAMILY_ROUNDED_CORNERS")
corner_extra_large_top = _define_token("SHAPE_FAMILY_ROUNDED_CORNERS")
corner_extra_extra_small = _define_token("SHAPE_FAMILY_ROUNDED_CORNERS")
corner_extra_small = _define_token("SHAPE_FAMILY_ROUNDED_CORNERS")
corner_extra_small_top = _define_token("SHAPE_FAMILY_ROUNDED_CORNERS")
corner_full = _define_token("SHAPE_FAMILY_CIRCULAR")
corner_large = _define_token("SHAPE_FAMILY_ROUNDED_CORNERS")
corner_large_end = _define_token("SHAPE_FAMILY_ROUNDED_CORNERS")
corner_large_start = _define_token("SHAPE_FAMILY_ROUNDED_CORNERS")
corner_large_top = _define_token("SHAPE_FAMILY_ROUNDED_CORNERS")
corner_medium = _define_token("SHAPE_FAMILY_ROUNDED_CORNERS")
corner_none = _define_token("SHAPE_FAMILY_ROUNDED_CORNERS")
corner_small = _define_token("SHAPE_FAMILY_ROUNDED_CORNERS")



from pytestqt.qtbot import QtBot

from material_ui.buttons import ElevatedButton


def test_ElevatedButton_basic_api(qtbot: QtBot):
    button = ElevatedButton()
    button.text = "Hi"

    def on_click() -> None:
        pass

    button.clicked.connect(on_click)
    qtbot.addWidget(button)



"""Filled text field component."""

from qtpy.QtCore import QMargins, QPoint

from material_ui._component import Component, effect
from material_ui.layout_basics import Row
from material_ui.shape import Line, Shape
from material_ui.text_fields._base_text_field import BaseTextField
from material_ui.tokens import md_comp_filled_text_field as tokens


class FilledTextField(BaseTextField):
    """Filled text field component."""

    _FLOATING_LABEL_POS = QPoint(18, 8)
    _RESTING_LABEL_POS = QPoint(16, 18)

    def __init__(self) -> None:
        super().__init__()

        self._background = Shape()
        self._background.corner_shape = tokens.container_shape
        self._background.color = tokens.container_color
        self._background.setParent(self)

        state_layer = Shape()
        state_layer.corner_shape = tokens.container_shape
        state_layer.visible = self.hovered
        state_layer.color = tokens.hover_state_layer_color
        state_layer.opacity = tokens.hover_state_layer_opacity
        self._background.overlay_widget(state_layer)

        self._active_indicator = Line()
        self._active_indicator.setParent(self._background)

        self._floating_label.setParent(self._background)
        self._floating_label.font_family = tokens.label_text_font
        self._floating_label.font_size = tokens.label_text_populated_size
        self._floating_label.font_weight = tokens.label_text_weight

        self._resting_label.setParent(self._background)
        self._resting_label.font_family = tokens.label_text_font
        self._resting_label.font_size = tokens.label_text_size
        self._resting_label.font_weight = tokens.label_text_weight

        row = Row()
        row.gap = 16
        row.margins = QMargins(16, 8, 16, 8)
        self.overlay_widget(row)

        # Use a wrapper to use the sx property, which Qt will propagate
        # to children by default.
        line_edit_wrapper = Component()
        line_edit_wrapper.sx = {
            "color": tokens.input_text_color,
            "font-family": tokens.input_text_font,
            "font-size": tokens.input_text_size,
            "font-weight": tokens.input_text_weight,
            "margin-top": "22px",
        }
        line_edit_wrapper.overlay_widget(self._line_edit)
        row.add_widget(line_edit_wrapper)

    @effect(Component.size)
    def _apply_size(self) -> None:
        self._background.resize(
            self.size().shrunkBy(QMargins(0, self._TOP_SPACE, 0, 0)),
        )
        self._background.move(0, self._TOP_SPACE)
        self._active_indicator.setFixedWidth(self._background.width())

    @effect(Component.size, Component.focused)
    def _refresh_active_indicator(self) -> None:
        if self.focused:
            self._active_indicator.color = tokens.focus_active_indicator_color
            self._active_indicator.thickness = tokens.focus_active_indicator_thickness
        else:
            self._active_indicator.color = tokens.active_indicator_color
            self._active_indicator.thickness = tokens.active_indicator_height
        # TODO: use an overlay layout that will just handle anchoring to
        #   bottom of background
        self._active_indicator.move(
            0,
            self._background.height() - self._active_indicator.resolved_thickness(),
        )

    @effect(Component.focused)
    def _refresh_label(self) -> None:
        color = (
            tokens.focus_label_text_color if self.focused else tokens.label_text_color
        )
        self._floating_label.sx = {**self._floating_label.sx, "color": color}
        self._resting_label.sx = {**self._resting_label.sx, "color": color}



"""Design tokens for md.comp.navigation-drawer."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

active_indicator_color = _define_token("md.sys.color.secondary-container")
active_indicator_height = _define_token(56)
active_indicator_shape = _define_token("md.sys.shape.corner.full")
active_indicator_width = _define_token(336)
active_focus_icon_color = _define_token("md.sys.color.on-secondary-container")
active_focus_label_text_color = _define_token("md.sys.color.on-secondary-container")
active_focus_state_layer_color = _define_token("md.sys.color.on-secondary-container")
active_hover_icon_color = _define_token("md.sys.color.on-secondary-container")
active_hover_label_text_color = _define_token("md.sys.color.on-secondary-container")
active_hover_state_layer_color = _define_token("md.sys.color.on-secondary-container")
active_icon_color = _define_token("md.sys.color.on-secondary-container")
active_label_text_color = _define_token("md.sys.color.on-secondary-container")
active_label_text_weight = _define_token("md.sys.typescale.label-large.weight.prominent")
active_pressed_icon_color = _define_token("md.sys.color.on-secondary-container")
active_pressed_label_text_color = _define_token("md.sys.color.on-secondary-container")
active_pressed_state_layer_color = _define_token("md.sys.color.on-secondary-container")
bottom_container_shape = _define_token("md.sys.shape.corner.large.top")
container_color = _define_token("md.sys.color.surface")
container_height = _define_token(1.0)
container_shape = _define_token("md.sys.shape.corner.large.end")
container_surface_tint_layer_color = _define_token("md.sys.color.surface-tint")
container_width = _define_token(360)
divider_color = _define_token("md.sys.color.outline")
focus_indicator_color = _define_token("md.sys.color.secondary")
focus_indicator_outline_offset = _define_token("md.sys.state.focus-indicator.inner-offset")
focus_indicator_thickness = _define_token("md.sys.state.focus-indicator.thickness")
focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
headline_color = _define_token("md.sys.color.on-surface-variant")
headline_font = _define_token("md.sys.typescale.title-small.font")
headline_line_height = _define_token("md.sys.typescale.title-small.line-height")
headline_size = _define_token("md.sys.typescale.title-small.size")
headline_tracking = _define_token("md.sys.typescale.title-small.tracking")
headline_weight = _define_token("md.sys.typescale.title-small.weight")
hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
icon_size = _define_token(24)
inactive_focus_icon_color = _define_token("md.sys.color.on-surface")
inactive_focus_label_text_color = _define_token("md.sys.color.on-surface")
inactive_focus_state_layer_color = _define_token("md.sys.color.on-surface")
inactive_hover_icon_color = _define_token("md.sys.color.on-surface")
inactive_hover_label_text_color = _define_token("md.sys.color.on-surface")
inactive_hover_state_layer_color = _define_token("md.sys.color.on-surface")
inactive_icon_color = _define_token("md.sys.color.on-surface-variant")
inactive_label_text_color = _define_token("md.sys.color.on-surface-variant")
inactive_pressed_icon_color = _define_token("md.sys.color.on-surface")
inactive_pressed_label_text_color = _define_token("md.sys.color.on-surface")
inactive_pressed_state_layer_color = _define_token("md.sys.color.on-secondary-container")
label_text_font = _define_token("md.sys.typescale.label-large.font")
label_text_line_height = _define_token("md.sys.typescale.label-large.line-height")
label_text_size = _define_token("md.sys.typescale.label-large.size")
label_text_tracking = _define_token("md.sys.typescale.label-large.tracking")
label_text_weight = _define_token("md.sys.typescale.label-large.weight")
large_badge_label_color = _define_token("md.sys.color.on-surface-variant")
large_badge_label_font = _define_token("md.sys.typescale.label-large.font")
large_badge_label_line_height = _define_token("md.sys.typescale.label-large.line-height")
large_badge_label_size = _define_token("md.sys.typescale.label-large.size")
large_badge_label_tracking = _define_token("md.sys.typescale.label-large.tracking")
large_badge_label_weight = _define_token("md.sys.typescale.label-large.weight")
modal_container_color = _define_token("md.sys.color.surface-container-low")
modal_container_elevation = _define_token("md.sys.elevation.level1")
pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
scrim_color = _define_token("md.ref.palette.neutral-variant20")
scrim_opacity = _define_token(0.4)
standard_container_color = _define_token("md.sys.color.surface")
standard_container_elevation = _define_token("md.sys.elevation.level0")



"""Switch component."""

from qtpy.QtCore import QEasingCurve, QRect
from qtpy.QtGui import QColor

from material_ui._component import Component, Signal, effect, use_state
from material_ui.shape import Shape
from material_ui.tokens import md_comp_switch as tokens

_UNSELECTED_TRACK_OUTLINE_COLOR = "#79747E"
_UNSELECTED_TRACK_COLOR = QColor("#E6E0E9")

_TRACK_WIDTH = 52
_TRACK_HEIGHT = 32
_STATE_LAYER_MARGIN = 4
_STATE_LAYER_SIZE = 40
_SWITCH_WIDTH = _TRACK_WIDTH + _STATE_LAYER_MARGIN * 2
_SWITCH_HEIGHT = _TRACK_HEIGHT + _STATE_LAYER_MARGIN * 2
_TRACK_OUTLINE_WIDTH = 2
_UNSELECTED_HANDLE_WIDTH = 16
_PRESSED_HANDLE_WIDTH = 28
_SELECTED_HANDLE_WIDTH = 24

_TRACK_GEOMETRY = QRect(
    _STATE_LAYER_MARGIN, _STATE_LAYER_MARGIN, _TRACK_WIDTH, _TRACK_HEIGHT
)
_UNSELECTED_HANDLE_GEOMETRY = QRect(
    _STATE_LAYER_MARGIN + (_TRACK_HEIGHT - _UNSELECTED_HANDLE_WIDTH) // 2,
    _STATE_LAYER_MARGIN + (_TRACK_HEIGHT - _UNSELECTED_HANDLE_WIDTH) // 2,
    _UNSELECTED_HANDLE_WIDTH,
    _UNSELECTED_HANDLE_WIDTH,
)
_UNSELECTED_PRESSED_HANDLE_GEOMETRY = QRect(
    _STATE_LAYER_MARGIN + _TRACK_OUTLINE_WIDTH,
    _STATE_LAYER_MARGIN + _TRACK_OUTLINE_WIDTH,
    _PRESSED_HANDLE_WIDTH,
    _PRESSED_HANDLE_WIDTH,
)
_SELECTED_PRESSED_HANDLE_GEOMETRY = QRect(
    _STATE_LAYER_MARGIN + _TRACK_WIDTH - _PRESSED_HANDLE_WIDTH - _TRACK_OUTLINE_WIDTH,
    _STATE_LAYER_MARGIN + _TRACK_OUTLINE_WIDTH,
    _PRESSED_HANDLE_WIDTH,
    _PRESSED_HANDLE_WIDTH,
)
_SELECTED_HANDLE_GEOMETRY = QRect(
    _STATE_LAYER_MARGIN
    + _TRACK_WIDTH
    - _SELECTED_HANDLE_WIDTH
    - (_TRACK_HEIGHT - _SELECTED_HANDLE_WIDTH) // 2,
    _STATE_LAYER_MARGIN + (_TRACK_HEIGHT - _SELECTED_HANDLE_WIDTH) // 2,
    _SELECTED_HANDLE_WIDTH,
    _SELECTED_HANDLE_WIDTH,
)
_UNSELECTED_STATE_LAYER_GEOMETRY = QRect(
    0,
    0,
    _STATE_LAYER_SIZE,
    _STATE_LAYER_SIZE,
)
_SELECTED_STATE_LAYER_GEOMETRY = QRect(
    _SWITCH_WIDTH - _SWITCH_HEIGHT,
    0,
    _STATE_LAYER_SIZE,
    _STATE_LAYER_SIZE,
)


class Switch(Component):
    """Switches toggle the selection of an item on or off."""

    selected = use_state(False)
    disabled = use_state(False)

    # Create states for the animated properties.
    _handle_geometry = use_state(
        _UNSELECTED_HANDLE_GEOMETRY,
        transition=100,
        easing=QEasingCurve.Type.InOutCubic,
    )
    _track_color = use_state(
        _UNSELECTED_TRACK_COLOR,
        # Shorter than the handle geometry animation to draw more
        # attention to the handle.
        transition=70,
        easing=QEasingCurve.Type.InOutCubic,
    )

    on_change: Signal[bool]
    """Emitted when the user toggles the switch."""

    def __init__(self) -> None:
        super().__init__()

        self.setFixedSize(_SWITCH_WIDTH, _SWITCH_HEIGHT)
        self.clicked.connect(self._on_click)
        self.should_propagate_click = False

        self._track = Shape()
        self._track.corner_shape = tokens.track_shape
        self._track.setGeometry(_TRACK_GEOMETRY)
        self._track.setParent(self)

        self._state_layer = Shape()
        self._state_layer.color = tokens.unselected_focus_state_layer_color
        self._state_layer.opacity = tokens.unselected_focus_state_layer_opacity
        self._state_layer.corner_shape = tokens.state_layer_shape
        self._state_layer.visible = self.hovered
        self._state_layer.setParent(self)

        self._handle = Shape()
        self._handle.corner_shape = tokens.handle_shape
        self._handle.setParent(self)

    def _on_click(self) -> None:
        """Handle click events to toggle the switch."""
        new_value = not self.selected
        self.selected = new_value
        self.on_change.emit(new_value)

    @effect(_handle_geometry)
    def _apply_handle_geometry(self) -> None:
        # TODO: make geometry a property of shape? even though conflict with qt property?
        self._handle.setGeometry(self._handle_geometry)

    @effect(selected)
    def _apply_track_color(self) -> None:
        self._track.color = (
            tokens.selected_track_color
            if self.selected
            else tokens.unselected_track_color
        )

    @effect(selected, Component.hovered)
    def _apply_handle_color(self) -> None:
        self._handle.color = (
            tokens.selected_hover_handle_color
            if self.selected and self.hovered
            else tokens.selected_handle_color
            if self.selected
            else tokens.unselected_hover_handle_color
            if self.hovered
            else tokens.unselected_handle_color
        )

    @effect(selected, Component.pressed, Component.hovered)
    def _refresh_shapes(self) -> None:
        selected_border = (
            f"{_TRACK_OUTLINE_WIDTH}px solid {_UNSELECTED_TRACK_OUTLINE_COLOR}"
        )
        self._track.sx = {
            **self._track.sx,
            "border": selected_border if not self.selected else "none",
        }

        self._handle_geometry = (
            _SELECTED_PRESSED_HANDLE_GEOMETRY
            if self.selected and self.pressed
            else _UNSELECTED_PRESSED_HANDLE_GEOMETRY
            if self.pressed
            else _SELECTED_HANDLE_GEOMETRY
            if self.selected
            else _UNSELECTED_HANDLE_GEOMETRY
        )

        self._state_layer.setGeometry(
            _SELECTED_STATE_LAYER_GEOMETRY
            if self.selected
            else _UNSELECTED_STATE_LAYER_GEOMETRY,
        )



"""Main app component."""

from material_ui._component import Component, Signal, effect, use_state
from material_ui.icon import Icon
from material_ui.layout_basics import Row, Stack
from material_ui.progress_indicators.linear_progress import LinearProgress
from material_ui.shape import Shape
from material_ui.tokens import md_sys_color, md_sys_shape
from material_ui.typography import Typography
from qtpy.QtCore import QMargins, Qt, QTimer
from qtpy.QtMultimedia import QAudioOutput, QMediaPlayer
from qtpy.QtWidgets import QApplication, QSizePolicy

songs = [
    "https://www.freecol.org/images/fearless-sailors.ogg",
    "https://www.freecol.org/images/founders.ogg",
    "https://www.freecol.org/images/settlers-routine.ogg",
    "https://www.freecol.org/images/sunrise.ogg",
    "https://www.freecol.org/images/tailwind.ogg",
    "https://www.freecol.org/images/musicbox.ogg",
    "https://www.freecol.org/images/FreeCol-opening.ogg",
    "https://www.freecol.org/images/FreeCol-menu.ogg",
]


class PlaybackControls(Component):
    on_click_skip_previous: Signal
    on_click_skip_next: Signal
    on_click_play_pause: Signal
    is_playing = use_state(False)
    is_loading = use_state(False)

    def __init__(self) -> None:
        super().__init__()

        self.setMinimumWidth(290)

        background = Shape()
        background.corner_shape = md_sys_shape.corner_extra_large
        background.color = md_sys_color.surface_container

        stack = Stack(
            alignment=Qt.AlignmentFlag.AlignCenter,
            margins=QMargins(20, 20, 20, 20),
            gap=15,
        )

        self._track_name_label = Typography()
        self._track_name_label.variant = "title-medium"
        self._track_name_label.alignment = Qt.AlignmentFlag.AlignCenter
        self._track_name_label.text = "Track Name"
        stack.add_widget(self._track_name_label)

        buttons_row = Row()
        buttons_row.alignment = Qt.AlignmentFlag.AlignCenter
        buttons_row.gap = 30

        self._skip_previous_button = Icon()
        self._skip_previous_button.icon_name = "skip_previous"
        self._skip_previous_button.clicked.connect(self.on_click_skip_previous)
        buttons_row.add_widget(self._skip_previous_button)

        self._play_pause_button = Icon()
        self._play_pause_button.clicked.connect(self.on_click_play_pause)
        buttons_row.add_widget(self._play_pause_button)

        self._skip_next_button = Icon()
        self._skip_next_button.icon_name = "skip_next"
        self._skip_next_button.clicked.connect(self.on_click_skip_next)
        buttons_row.add_widget(self._skip_next_button)

        stack.add_widget(buttons_row)

        self._seek_bar = Shape()
        self._seek_bar.color = md_sys_color.primary
        self._seek_bar.corner_shape = md_sys_shape.corner_full
        self._seek_bar.setFixedHeight(11)
        self._seek_bar.setSizePolicy(
            QSizePolicy.Policy.Expanding,
            QSizePolicy.Policy.Fixed,
        )
        stack.add_widget(self._seek_bar)

        self._loading_bar = LinearProgress()
        self._loading_bar.indeterminate = True
        stack.add_widget(self._loading_bar)

        background.overlay_widget(stack)
        self.overlay_widget(background)

    @effect(is_loading)
    def _apply_loading_state(self) -> None:
        if self.is_loading:
            self._seek_bar.visible = False
            self._loading_bar.show()
        else:
            self._seek_bar.visible = True
            self._loading_bar.hide()

    @effect(is_playing)
    def _apply_play_pause_icon(self) -> None:
        if self.is_playing:
            self._play_pause_button.icon_name = "pause"
        else:
            self._play_pause_button.icon_name = "play_arrow"


class MusicPlayerApp(Component):
    _is_playing = use_state(False)
    _is_loading = use_state(False)
    _active_song_index = use_state(0)

    def __init__(self) -> None:
        super().__init__()
        self.sx = {"background-color": md_sys_color.background}

        stack = Stack(
            alignment=Qt.AlignmentFlag.AlignCenter,
            margins=QMargins(20, 20, 20, 20),
        )

        playback_controls = PlaybackControls()
        playback_controls.is_playing = self._is_playing
        playback_controls.is_loading = self._is_loading
        playback_controls.on_click_skip_previous.connect(self._skip_previous)
        playback_controls.on_click_skip_next.connect(self._skip_next)
        playback_controls.on_click_play_pause.connect(self._toggle_play_pause)
        stack.add_widget(playback_controls)

        self.overlay_widget(stack)

        self._media_player = QMediaPlayer()
        self._media_player.setParent(self)
        audio_output = QAudioOutput()
        audio_output.setParent(self._media_player)
        audio_output.setVolume(100.0)
        self._media_player.setAudioOutput(audio_output)
        self._media_player.mediaStatusChanged.connect(
            self._on_media_player_media_status_changed,
        )

    def _on_media_player_media_status_changed(
        self,
        status: QMediaPlayer.MediaStatus,
    ) -> None:
        # Qt emits a LoadedMedia status with the old source when
        # switching sources - filter these events out.
        is_correct_song = (
            self._media_player.source().toString() == songs[self._active_song_index]
        )
        if status == QMediaPlayer.MediaStatus.LoadedMedia and is_correct_song:
            # Media loaded, we can start playing.
            def delayed_play() -> None:
                self._is_loading = False
                self._is_playing = True

            # Delay otherwise Qt won't actually play anything.
            QTimer.singleShot(0, delayed_play)

    def _toggle_play_pause(self) -> None:
        self._is_playing = not self._is_playing

    def _skip_previous(self) -> None:
        self._active_song_index = (
            self._active_song_index - 1
            if self._active_song_index > 0
            else len(songs) - 1
        )

    def _skip_next(self) -> None:
        self._active_song_index = (
            self._active_song_index + 1
            if self._active_song_index < len(songs) - 1
            else 0
        )

    @effect(_active_song_index)
    def _load_song(self) -> None:
        self._media_player.setSource(songs[self._active_song_index])
        self._is_loading = True
        self._is_playing = False

    @effect(_is_playing)
    def _apply_media_player_play_state(self) -> None:
        if self._is_playing:
            self._media_player.play()
        else:
            self._media_player.pause()


def main() -> None:
    app = QApplication()
    window = MusicPlayerApp()
    window.show()
    app.exec()


if __name__ == "__main__":
    main()



from pytest_mock import MockerFixture
from pytestqt.qtbot import QtBot
from qtpy.QtCore import Qt
from qtpy.QtWidgets import QSizePolicy

from material_ui.buttons._button_base import ButtonBase


def test_ButtonBase_click_is_fired(qtbot: QtBot, mocker: MockerFixture):
    click_stub = mocker.stub()
    button = ButtonBase()
    button.clicked.connect(click_stub)
    qtbot.addWidget(button)
    qtbot.mouseClick(button, Qt.MouseButton.LeftButton)
    assert click_stub.call_count == 1


def test_ButtonBase_click_right_click_does_nothing(qtbot: QtBot, mocker: MockerFixture):
    click_stub = mocker.stub()
    button = ButtonBase()
    button.clicked.connect(click_stub)
    qtbot.addWidget(button)
    qtbot.mouseClick(button, Qt.MouseButton.RightButton)
    assert click_stub.call_count == 0


def test_ButtonBase_hovered(qtbot: QtBot):
    button = ButtonBase()
    qtbot.addWidget(button)
    c1 = button._state_layer.color
    button.hovered = True
    c2 = button._state_layer.color
    assert c1 != c2


def test_ButtonBase_size_hint_with_texts(qtbot: QtBot):
    button = ButtonBase()
    button.setSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Minimum)
    button.text = "Hello world"
    qtbot.addWidget(button)
    size_hint = button.sizeHint()
    assert 100 < size_hint.width() < 130
    assert 40 < size_hint.height() < 70

    button.text = "Hi"
    qtbot.wait(1)
    size_hint2 = button.sizeHint()
    # Shorter text should be narrower.
    assert size_hint2.width() < size_hint.width()



"""Design tokens for md.comp.time-picker."""

# Auto generated by _token_converter.py
# Do not edit this file directly.

from material_ui.tokens._utils import define_token as _define_token

clock_dial_color = _define_token("md.sys.color.surface-container-highest")
clock_dial_color_ignore = _define_token("md.sys.color.on-surface-variant")
clock_dial_container_size = _define_token(256)
clock_dial_label_text_font = _define_token("md.sys.typescale.body-large.font")
clock_dial_label_text_line_height = _define_token("md.sys.typescale.body-large.line-height")
clock_dial_label_text_size = _define_token("md.sys.typescale.body-large.size")
clock_dial_label_text_tracking = _define_token("md.sys.typescale.body-large.tracking")
clock_dial_label_text_weight = _define_token("md.sys.typescale.body-large.weight")
clock_dial_selected_label_text_color = _define_token("md.sys.color.on-primary")
clock_dial_selector_center_container_color = _define_token("md.sys.color.primary")
clock_dial_selector_center_container_shape = _define_token("md.sys.shape.corner.full")
clock_dial_selector_center_container_size = _define_token(8)
clock_dial_selector_handle_container_color = _define_token("md.sys.color.primary")
clock_dial_selector_handle_container_shape = _define_token("md.sys.shape.corner.full")
clock_dial_selector_handle_container_size = _define_token(48)
clock_dial_selector_track_container_color = _define_token("md.sys.color.primary")
clock_dial_selector_track_container_width = _define_token(2)
clock_dial_shape = _define_token("md.sys.shape.corner.full")
clock_dial_shape_ignore = _define_token("md.sys.shape.corner.full")
clock_dial_unselected_label_text_color = _define_token("md.sys.color.on-surface")
container_color = _define_token("md.sys.color.surface-container-high")
container_elevation = _define_token("md.sys.elevation.level3")
container_shape = _define_token("md.sys.shape.corner.extra-large")
headline_color = _define_token("md.sys.color.on-surface-variant")
headline_font = _define_token("md.sys.typescale.label-medium.font")
headline_line_height = _define_token("md.sys.typescale.label-medium.line-height")
headline_size = _define_token("md.sys.typescale.label-medium.size")
headline_tracking = _define_token("md.sys.typescale.label-medium.tracking")
headline_weight = _define_token("md.sys.typescale.label-medium.weight")
period_selector_container_shape = _define_token("md.sys.shape.corner.small")
period_selector_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
period_selector_horizontal_container_height = _define_token(38)
period_selector_horizontal_container_width = _define_token(216)
period_selector_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
period_selector_label_text_font = _define_token("md.sys.typescale.title-medium.font")
period_selector_label_text_line_height = _define_token("md.sys.typescale.title-medium.line-height")
period_selector_label_text_size = _define_token("md.sys.typescale.title-medium.size")
period_selector_label_text_tracking = _define_token("md.sys.typescale.title-medium.tracking")
period_selector_label_text_weight = _define_token("md.sys.typescale.title-medium.weight")
period_selector_outline_color = _define_token("md.sys.color.outline")
period_selector_outline_width = _define_token(1)
period_selector_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
period_selector_selected_container_color = _define_token("md.sys.color.tertiary-container")
period_selector_selected_focus_label_text_color = _define_token("md.sys.color.on-tertiary-container")
period_selector_selected_focus_state_layer_color = _define_token("md.sys.color.on-tertiary-container")
period_selector_selected_hover_label_text_color = _define_token("md.sys.color.on-tertiary-container")
period_selector_selected_hover_state_layer_color = _define_token("md.sys.color.on-tertiary-container")
period_selector_selected_label_text_color = _define_token("md.sys.color.on-tertiary-container")
period_selector_selected_pressed_label_text_color = _define_token("md.sys.color.on-tertiary-container")
period_selector_selected_pressed_state_layer_color = _define_token("md.sys.color.on-tertiary-container")
period_selector_unselected_focus_label_text_color = _define_token("md.sys.color.on-surface-variant")
period_selector_unselected_focus_state_layer_color = _define_token("md.sys.color.on-surface-variant")
period_selector_unselected_hover_label_text_color = _define_token("md.sys.color.on-surface-variant")
period_selector_unselected_hover_state_layer_color = _define_token("md.sys.color.on-surface-variant")
period_selector_unselected_label_text_color = _define_token("md.sys.color.on-surface-variant")
period_selector_unselected_pressed_label_text_color = _define_token("md.sys.color.on-surface-variant")
period_selector_unselected_pressed_state_layer_color = _define_token("md.sys.color.on-surface-variant")
period_selector_vertical_container_height = _define_token(80)
period_selector_vertical_container_width = _define_token(52)
surface_tint_layer_color = _define_token("md.sys.color.surface-tint")
time_selector_24h_vertical_container_width = _define_token(114)
time_selector_container_height = _define_token(80)
time_selector_container_shape = _define_token("md.sys.shape.corner.small")
time_selector_container_width = _define_token(96)
time_selector_focus_state_layer_opacity = _define_token("md.sys.state.focus.state-layer-opacity")
time_selector_hover_state_layer_opacity = _define_token("md.sys.state.hover.state-layer-opacity")
time_selector_label_text_font = _define_token("md.sys.typescale.display-large.font")
time_selector_label_text_line_height = _define_token("md.sys.typescale.display-large.line-height")
time_selector_label_text_size = _define_token("md.sys.typescale.display-large.size")
time_selector_label_text_tracking = _define_token("md.sys.typescale.display-large.tracking")
time_selector_label_text_weight = _define_token("md.sys.typescale.display-large.weight")
time_selector_pressed_state_layer_opacity = _define_token("md.sys.state.pressed.state-layer-opacity")
time_selector_selected_container_color = _define_token("md.sys.color.primary-container")
time_selector_selected_focus_label_text_color = _define_token("md.sys.color.on-primary-container")
time_selector_selected_focus_state_layer_color = _define_token("md.sys.color.on-primary-container")
time_selector_selected_hover_label_text_color = _define_token("md.sys.color.on-primary-container")
time_selector_selected_hover_state_layer_color = _define_token("md.sys.color.on-primary-container")
time_selector_selected_label_text_color = _define_token("md.sys.color.on-primary-container")
time_selector_selected_pressed_label_text_color = _define_token("md.sys.color.on-primary-container")
time_selector_selected_pressed_state_layer_color = _define_token("md.sys.color.on-primary-container")
time_selector_separator_color = _define_token("md.sys.color.on-surface")
time_selector_separator_font = _define_token("md.sys.typescale.display-large.font")
time_selector_separator_line_height = _define_token("md.sys.typescale.display-large.line-height")
time_selector_separator_size = _define_token("md.sys.typescale.display-large.size")
time_selector_separator_tracking = _define_token("md.sys.typescale.display-large.tracking")
time_selector_separator_weight = _define_token("md.sys.typescale.display-large.weight")
time_selector_unselected_container_color = _define_token("md.sys.color.surface-container-highest")
time_selector_unselected_focus_label_text_color = _define_token("md.sys.color.on-surface")
time_selector_unselected_focus_state_layer_color = _define_token("md.sys.color.on-surface")
time_selector_unselected_hover_label_text_color = _define_token("md.sys.color.on-surface")
time_selector_unselected_hover_state_layer_color = _define_token("md.sys.color.on-surface")
time_selector_unselected_label_text_color = _define_token("md.sys.color.on-surface")
time_selector_unselected_pressed_label_text_color = _define_token("md.sys.color.on-surface")
time_selector_unselected_pressed_state_layer_color = _define_token("md.sys.color.on-surface")



"""Utilities for internal default fonts."""

import asyncio
from functools import cache, partial
from hashlib import md5
from pathlib import Path
from tempfile import gettempdir

import httpx
from qtpy.QtGui import QFontDatabase

_FONT_URLS = [
    "https://raw.githubusercontent.com/google/material-design-icons/refs/heads/master/variablefont/MaterialSymbolsOutlined[FILL,GRAD,opsz,wght].ttf",
    "https://raw.githubusercontent.com/google/material-design-icons/refs/heads/master/variablefont/MaterialSymbolsRounded[FILL,GRAD,opsz,wght].ttf",
    "https://raw.githubusercontent.com/google/material-design-icons/refs/heads/master/variablefont/MaterialSymbolsSharp[FILL,GRAD,opsz,wght].ttf",
    # Regular
    "https://fonts.gstatic.com/s/roboto/v47/KFOmCnqEu92Fr1Me5WZLCzYlKw.ttf",
    # Italic
    # https://fonts.gstatic.com/s/roboto/v47/KFOkCnqEu92Fr1Mu52xPKTM1K9nz.ttf
]


@cache
def install_default_fonts() -> bool:
    """Apply the material fonts to the QFontDatabase.

    Returns:
        Whether all fonts were successfully installed.
    """
    file_paths = asyncio.run(_download_all_fonts())
    font_ids = [
        QFontDatabase.addApplicationFont(str(font_path))
        for font_path in filter(None, file_paths)
    ]
    return all(file_paths) and all(font_id != -1 for font_id in font_ids)


async def _download_all_fonts() -> list[Path | None]:
    async with httpx.AsyncClient() as client:
        return await asyncio.gather(*map(partial(_download_font, client), _FONT_URLS))


async def _download_font(
    client: httpx.AsyncClient,
    url: str,
    *,
    no_cache: bool = False,
) -> Path | None:
    """Fetch a font from a URL and save to disk.

    Args:
        client: The HTTP client to use.
        url: The URL of the font to fetch.
        no_cache: If True, ignore the cached file.

    Returns:
        Path to the downloaded font file on disk, or None if it failed.
    """
    file_path = _get_cache_path_for_url(url)

    if no_cache or not file_path.exists():
        resp = await client.get(url)
        if resp.status_code != httpx.codes.OK:
            return None
        file_path.parent.mkdir(parents=True, exist_ok=True)
        with file_path.open("wb") as f:
            f.write(resp.content)

    return file_path


def _get_cache_path_for_url(url: str) -> Path:
    return _get_font_cache_dir() / md5(url.encode(), usedforsecurity=False).hexdigest()


def _get_font_cache_dir() -> Path:
    """Get the path to the font cache directory."""
    return Path(gettempdir()) / "qt-material-ui" / "font_cache"
